package Newslash::Command::useradd;
use Mojo::Base 'Newslash::Command';

use Mojo::Util 'getopt';
use Data::Dumper;

has description => 'Add new user';
has usage => sub { shift->extract_usage };

has experimental => 1;
has danger => 0;

sub run {
    my ($self, @args) = @_;
    return if !$self->check_force_option(\@args);

    my $app = $self->app;
    my $seclev = 1;
    my $author = 0;
    my $password = "";

    getopt \@args,
      'u|username=s' => \my $nickname,
      'e|email=s'    => \my $email,
      'p|password=s' => \$password,
      'seclev=i'     => \$seclev,
      'author=i'     => \$author;

    if (!$nickname || !$email) {
        $app->log->error("useradd: nickname or email not given!");
        return;
    }

    my $users = $app->model('users');
    if (!$password) {
        $password = $users->passwords->generate_random_password;
        $app->log->info("useradd: generate random password - $password");
    }

    my $uid = $users->create($nickname, $email, $password);
    if ($uid) {
        $app->log->info("useradd: create new user '$nickname'. uid is $uid");
    }
    else {
        my $err = $users->last_error;
        $app->log->error("useradd: create new user '$nickname' failed - $err");
    }

    if ($seclev || $author) {
        my $rs = $users->update(uid => $uid,
                                seclev => $seclev,
                                author => $author);
        $app->log->info("useradd: update user info done ($rs)");
    }
}

1;

=encoding utf8

=head1 NAME

Newslash::Command::useradd - newslash administer command (useradd)

=head1 SYNOPSIS

  Usage: APPLICATION useradd [OPTIONS]

      ./newslash_web useradd -u foobar -e foo@example.com -p hogehoge

    Options:
      -u, --username <username>  user name (nickname)
      -e, --email <email>        email address
      -p, --password <password>  password
      --seclev <seclev>          seclev
      --author <0/1>             set author flag

=head1 DESCRIPTION

  L<Newslash::Command::useradd> inserts new user to newslash's user database.

=head1 ATTRIBUTES

L<Newslash::Command::useradd> inherits all attributes from
L<Newslash::Command> and implements the following new ones.

=head2 description

  my $description = $daemon->description;
  $daemon         = $daemon->description('Foo');

Short description of this command, used for the command list.

=head2 usage

  my $usage = $daemon->usage;
  $daemon   = $daemon->usage('Foo');

Usage information for this command, used for the help screen.

=head1 METHODS

L<Newslash::Command::useradd> inherits all methods from
L<Newslash::Command> and implements the following new ones.

=head2 run

  $daemon->run(@ARGV);

Run this command.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>.

=cut
