package Newslash::Model::ADCodes;
use Newslash::Model::Base -base;
use Data::Dumper;

sub on_start_up {
    my $self = shift;

    # create tables
    return 1 if $self->check_readonly;

    my $sqls = {
                ns_ad_codes => <<"EOSQL",
CREATE TABLE IF NOT EXISTS ns_ad_codes (
  ad_id        mediumint(8) unsigned NOT NULL PRIMARY KEY AUTO_INCREMENT,
  name         varchar(191) UNIQUE NOT NULL,
  description  varchar(255) DEFAULT "",
  status       enum('enabled', 'disabled') DEFAULT 'enabled',
  content      mediumtext,
  last_update  timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
) DEFAULT CHARSET=utf8mb4
EOSQL
               };

    my $dbh = $self->connect_db;
    for my $table_name (keys %$sqls) {
        if (!$self->table_exists($table_name)) {
            my $rs = $dbh->do($sqls->{$table_name}, undef);
            if ($rs) {
                $self->warn("create table: $table_name");
            }
            else {
                $self->warn("cannot create table: $table_name");
            }
        }
    }
    $self->disconnect_db;

    return 1;
}

sub select {
    my $self = shift;
    my $params = {@_};

    my $unique_keys = { id => "ns_ad_codes.ad_id",
                        ad_id => "ns_ad_codes.ad_id",
                        name => "ns_ad_codes.name",
                      };
    my $keys = { last_update => "ns_ad_codes.last_update",
                 status => "ns_ad_codes.status",
               };
    my $timestamp = "ns_ad_codes.last_update";

    my ($where_clause, $where_values, $unique) = $self->build_where_clause(unique_keys => $unique_keys,
                                                                           keys => $keys,
                                                                           timestamp => $timestamp,
                                                                           params => $params);
    my ($limit_clause, $limit_values) = $self->build_limit_clause(params => $params);
    my ($orderby_clause, $orderby_values) = $self->build_order_by_clause(keys => $keys,
                                                                         params => $params);

    my @attrs;
    push @attrs, @$where_values, @$limit_values, @$orderby_values;

    my $dbh = $self->connect_db;
    my $sql = <<"EOSQL";
SELECT * FROM ns_ad_codes
    $where_clause
    $orderby_clause
    $limit_clause
EOSQL

    #warn($sql);
    #warn(Dumper(@attrs));

    my $sth = $dbh->prepare($sql);
    $sth->execute(@attrs);
    my $feeds = $sth->fetchall_arrayref({});

    if (!$feeds) {
        $self->disconnect_db();
        return;
    }
    if (@$feeds == 0) {
        $self->disconnect_db();
        return $unique ? undef : [];
    }

    return $feeds->[0] if $unique;
    return $feeds;
}

sub create {
    my $self = shift;
    my $params = {@_};
    my $name = $params->{name};
    return if !$name;

    my $dbh = $self->connect_db;
    my $sql = <<"EOSQL";
INSERT INTO ns_ad_codes
    (name, description, status, content, last_update)
  VALUES
    (?,    ?,           ?,      ?,           NOW()  )
EOSQL

    my $rs = $dbh->do($sql, undef,
                      $name,
                      $params->{description},
                      $params->{status} || 'enabled',
                      $params->{content}
                     );
    return if !$rs;
    my $id = $dbh->last_insert_id(undef, undef, undef, undef);
    return $id;
}

sub update {
    my $self = shift;
    my $params = {@_};
    my $ad_id = $params->{ad_id} || $params->{id};
    return if !$ad_id;

    my @cols;
    my @attrs;
    for my $k (qw|name description status content|) {
        if (defined $params->{$k}) {
            push @cols, "$k = ?";
            push @attrs, $params->{$k};
        }
    }
    my $columns = join(", ", @cols);
    my $sql = <<"EOSQL";
UPDATE ns_ad_codes 
  SET $columns
  WHERE ad_id = ?
EOSQL

    push @attrs, $ad_id;
    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, @attrs);
    return if !$rs;
    return $ad_id;
}

sub delete {
    my $self = shift;
    my $params = {@_};
    my $ad_id = $params->{id} || $params->{ad_id};
    return if !$ad_id;

    my $sql = <<"EOSQL";
DELETE FROM ns_ad_codes
  WHERE ad_id = ?
EOSQL

    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, $ad_id);
    return $rs;
}

sub export_all {
    my $self = shift;
    my $target = shift;

    if (ref($target) eq "SCALAR") {
        $$target = $self->export_json(table => "ns_ad_codes",
                                      sort_key => "name",
                                      exclude => [qw(ad_id
                                                     last_update
                                                   )],
                                     );
        return 1;
    }
    my $output = $self->export_json(file => $target,
                                    table => "ns_ad_codes",
                                    sort_key => "name",
                                    exclude => [qw(ad_id
                                                   last_update
                                                 )]
                                   );
    return if !$output;
    return 1;
}

sub import_all {
    my $self = shift;
    my $target = shift;

    my $output;

    if (ref($target) eq "SCALAR") {
        my $rs = $self->import_json(table => "ns_ad_codes",
                                    json => $$target,
                                    unique_key => "name",
                                    exclude => [qw(ad_id
                                                   last_update
                                                 )]
                                   );
        return $rs;
    }
    my $rs = $self->import_json(table => "ns_ad_codes",
                                file => $target,
                                unique_key => "name",
                                exclude => [qw(ad_id
                                               last_update
                                             )],
                               );
    return $rs;
}

1;
