package Newslash::Model::Events;
use Newslash::Model::Base -base;

use Mojo::JSON qw(to_json from_json);
use Encode qw(encode_utf8 decode_utf8);
use Mojo::Log;

use constant EVENTS_TABLE_NAME => 'ns_events';

sub key_definition {
    return {
            table => "ns_events",
            primary => "event_id",
            datetime => [qw(created_timestamp)],
            timestamp => "created_timestamp",
            other => [qw(event_type event_subtype
                         related_id source_user_id
                         priority active result)],
            aliases => { event_id => "id",
                         create_time => "created_timestamp",
                       }
           };
}

sub on_start_up {
    my $self = shift;

    # create tables
    return 1 if $self->check_readonly;

    my $table_name = EVENTS_TABLE_NAME;
    if (!$self->table_exists($table_name)) {
        my $dbh = $self->connect_db;

        my $sql = <<"EOSQL";
CREATE TABLE IF NOT EXISTS $table_name (
  event_id          mediumint(8) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
  event_type        varchar(32)           NOT NULL,
  event_subtype     varchar(32),
  related_id        mediumint(8) unsigned,
  source_user_id    mediumint(8) unsigned,
  created_timestamp timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  priority          tinyint(4)           DEFAULT '0',
  active            tinyint(4)           NOT NULL DEFAULT '1',
  result            tinyint(4)
) DEFAULT CHARSET=utf8mb4
EOSQL
        my $rs = $dbh->do($sql, undef);
        $self->disconnect_db;
        if ($rs) {
            return 1;
        }
        $self->warn("cannot create table: $table_name");
    }

    return 1;
}

sub select {
    my $self = shift;
    my $params = {@_};

    return $self->generic_select(parmas => $params);
}


sub create {
    my $self = shift @_;
    my $params = {@_};
    return if $self->check_readonly;

    my $event_type = $params->{event_type};
    return if !$event_type;

    my $table_name = EVENTS_TABLE_NAME;
    my $sql = <<"EOSQL";
INSERT INTO $table_name
    (event_type, event_subtype, related_id, source_user_id, priority, active)
  VALUES
    (?,          ?,             ?,          ?,              ?,        ?)
EOSQL

    my @values = ($event_type,
                  $params->{event_subtype},
                  $params->{related_id},
                  $params->{source_user_id},
                  $params->{priority},
                  defined $params->{active} ? $params->{active} : 1);
    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, @values);
    if (!$rs) {
        $self->set_error("insert failed", $dbh->{mysql_errorno});
        $self->disconnect_db;
        return;
    }
    my $event_id = $dbh->last_insert_id(undef, undef, undef, undef);
    $self->disconnect_db;

    return $event_id;
}

sub delete {
    my $self = shift @_;
    my $params = {@_};
    my $event_id = $params->{event_id};
    return if !$event_id;

    my $table_name = EVENTS_TABLE_NAME;
    my $sql = <<"EOSQL";
DELETE FROM $table_name WHERE event_id = ?
EOSQL

    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, $event_id);
    if (!defined $rs) {
        $self->set_error("delete failed", $dbh->{mysql_errorno});
        $self->disconnect_db;
        return;
    }
    $self->disconnect_db;

    return $rs;
}

sub update {
    my $self = shift;
    my $params = {@_};
    return if $self->check_readonly;

    my $id = $params->{event_id} || $params->{id};
    return if !$id;
    my @cl;
    my @val;
    if (defined $params->{active}) {
        push @cl, "active = ?";
        push @val, $params->{active};
    }
    if (defined $params->{result}) {
        push @cl, "result = ?";
        push @val, $params->{result};
    }
    if (defined $params->{priority}) {
        push @cl, "priority = ?";
        push @val, $params->{priority};
    }
    my $set_clause = join(", ", @cl);
    return if !$set_clause;
    push @val, $id;

    my $table = EVENTS_TABLE_NAME;
    my $sql = <<"EOSQL";
UPDATE $table SET $set_clause WHERE event_id = ?
EOSQL
    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, @val);
    if (!defined $rs) {
        $self->set_error("delete failed", $dbh->{mysql_errorno});
        $self->disconnect_db;
        return;
    }
    $self->disconnect_db;
    return $rs;
}

sub select {
    my $self = shift;
    return $self->generic_select(params => {@_});
}

sub oldest {
    my $self = shift;
    my $rs = $self->select(order_by => "created_timestamp", limit => 1, active => 1);
    if ($rs) {
        return $rs->[0];
    }
    return;
}

sub fetch {
    my ($self, $max_count) = @_;
    $max_count ||= 10;
    my $rs = $self->select(order_by => { DESC => "priority",
                                         DESC => "created_timestamp" },
                           limit => $max_count,
                           active => 1);
    if ($rs) {
        return $rs;
    }
    return;
}

sub delete_all {
    my $self = shift;
    my $table = EVENTS_TABLE_NAME;
    my $sql = "DELETE FROM $table";

    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql);
    if (!defined $rs) {
        $self->set_error("delete failed", $dbh->{mysql_errorno});
        $self->disconnect_db;
        return;
    }
    $self->disconnect_db;
    return $rs;
}

1;
