package Newslash::Model::Filters;
use Newslash::Model::Base -base;
use Data::Dumper;

sub key_definition {
    return {
            table => "ns_filters",
            primary => "filter_id",
            unique => "name",
            other => [qw(description message
                         status regexp ratio
                         threshold limit pathes fields
                       )],
            aliases => { id => "filter_id", }
           };
}

sub on_start_up {
    my $self = shift;

    # create tables
    return 1 if $self->check_readonly;

    my $sql = <<"EOSQL";
CREATE TABLE IF NOT EXISTS ns_filters (
  filter_id    mediumint(8) unsigned NOT NULL PRIMARY KEY AUTO_INCREMENT,
  name         varchar(191)          UNIQUE NOT NULL,
  description  varchar(255)          DEFAULT "",
  message      varchar(255)          DEFAULT "",
  status       enum('enabled', 'disabled') DEFAULT 'enabled',
  `regexp`     varchar(255)          DEFAULT "",
  ratio        float unsigned        DEFAULT "1.0",
  threshold    tinyint(3)            DEFAULT "0",
  `limit`      tinyint(3)            DEFAULT "0",
  pathes       mediumtext,
  fields       mediumtext
) DEFAULT CHARSET=utf8mb4
EOSQL

    my $dbh = $self->connect_db;
    my $table_name = "ns_filters";
    if (!$self->table_exists($table_name)) {
        my $rs = $dbh->do($sql, undef);
        if ($rs) {
            $self->warn("create table: $table_name");
        }
        else {
            $self->warn("cannot create table: $table_name");
        }
    }
    $self->disconnect_db;

    return 1;
}

sub _flatten_array {
    my ($self, $array) = @_;
    if ($array && ref($array) eq "ARRAY") {
        $array = join(" ", @$array);
    }
    return $array;
}

sub _restore_to_array {
    my ($self, $text) = @_;
    if ($text && !ref($text)) {
        $text = [split(/[, ]/, $text)];
    }
    return $text;
}

sub create {
    my $self = shift;
    my $params = {@_};

    $params->{pathes} = $self->_flatten_array($params->{pathes});
    $params->{fields} = $self->_flatten_array($params->{fields});

    my $rs = $self->generic_insert(params => $params);
    return if !defined $rs;

    return $rs;
}

sub select {
    my $self = shift;
    my $params = {@_};
    my $rs = $self->generic_select(params => $params);
    return if !defined $rs;

    if (ref($rs) eq "HASH") {
        $rs->{pathes} = $self->_restore_to_array($rs->{pathes});
        $rs->{fields} = $self->_restore_to_array($rs->{fields});
    }
    else {
        for my $item (@$rs) {
            $item->{pathes} = $self->_restore_to_array($item->{pathes});
            $item->{fields} = $self->_restore_to_array($item->{fields});
        }
    }

    return $rs;
}

sub update {
    my $self = shift;
    my $params = {@_};

    if (!$params->{filter_id}) {
        return;
    }
    my $id = $params->{filter_id};
    delete $params->{filter_id};

    $params->{pathes} = $self->_flatten_array($params->{pathes});
    $params->{fields} = $self->_flatten_array($params->{fields});

    return $self->generic_update(params => $params, where => {filter_id => $id});
}

sub delete {
    my $self = shift;
    my $params = {@_};
    return $self->generic_delete(params => $params);
}

sub export_all {
    my $self = shift;
    my $target = shift;

    if (ref($target) eq "SCALAR") {
        $$target = $self->export_json(table => "ns_filters",
                                      sort_key => "name",
                                      exclude => [],
                                     );
        return 1;
    }
    my $output = $self->export_json(file => $target,
                                    table => "ns_filters",
                                    sort_key => "name",
                                    exclude => []
                                   );
    return if !$output;
    return 1;
}

sub import_all {
    my $self = shift;
    my $target = shift;

    my $output;

    if (ref($target) eq "SCALAR") {
        my $rs = $self->import_json(table => "ns_filters",
                                    json => $$target,
                                    unique_key => "name",
                                    exclude => []
                                   );
        return $rs;
    }
    my $rs = $self->import_json(table => "ns_filters",
                                file => $target,
                                unique_key => "name",
                                exclude => [],
                               );
    return $rs;
}

1;
