package Newslash::Model::Users::Configs;
use Newslash::Model::Base -base;
use List::Util qw(any);

use JSON;

use constant CONFIG_TABLE_NAME => 'ns_users_config';
use constant CONFIG_SECTIONS => qw(info ui comment);

sub _check_and_create_table {
    my $self = shift;

    # create tables
    return 1 if $self->check_readonly;

    my $table_name = CONFIG_TABLE_NAME;
    if (!$self->table_exists($table_name)) {
        my $dbh = $self->connect_db;

        my $sql = <<"EOSQL";
CREATE TABLE IF NOT EXISTS $table_name (
  uid     mediumint(8) unsigned NOT NULL UNIQUE,
  configs TEXT
) DEFAULT CHARSET=utf8mb4
EOSQL
        my $rs = $dbh->do($sql, undef);
        $self->disconnect_db;
        if ($rs) {
            $self->warn("create table: $table_name");
        }
        else {
            $self->warn("cannot create table: $table_name");
        }
    }

    return 1;
}

sub select {
    my $self = shift;
    my $params = {@_};

    my $uid = $params->{id} || $params->{uid} || $params->{user_id};
    return if !$uid;

    my $rs = $self->generic_select(table => "ns_users_config",
                                   unique_keys => ["uid"],
                                   params => {uid => $uid});
    if (!$rs) {
        my $prefs = $self->new_instance_of('::Users::Prefs');
        return $prefs->to_config($uid);
    }
    my $configs = $rs->{configs};
    return JSON::from_json($configs) || {};
}

sub update {
    my $self = shift;
    return if $self->check_readonly;
    my $params = {@_};

    my $uid = $params->{id} || $params->{uid} || $params->{user_id};
    return if !$uid;

    my $cfg = $params->{config};
    # parameter mode
    if (!defined $params->{config}) {
        my $updated = 0;
        $cfg = $self->select(id => $uid);

        for my $section (CONFIG_SECTIONS) {
            for my $k (keys %{$params->{$section}}) {
                $cfg->{$section}->{$k} = $params->{$section}->{$k};
                $updated = 1;
            }
        }
        if (!$updated) {
            return $cfg;
        }
    }

    my $json = JSON::to_json($cfg);
    return if !$json;

    my $table_name = CONFIG_TABLE_NAME;
    my $sql = <<"EOSQL";
INSERT INTO $table_name (uid, configs) VALUES (?, ?)
  ON DUPLICATE KEY UPDATE configs = ?
EOSQL

    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, $uid, $json, $json);
    if (!$rs) {
        $self->disconnect_db;
        return;
    }

    # update `homepage` and `signature`
    my @set_clauses;
    my @sql_param;
    if (defined $cfg->{info}->{homepage}) {
        push @set_clauses, "homepage = ?";
        push @sql_param, $cfg->{homepage};
    }
    if (defined $cfg->{info}->{signature}) {
        push @set_clauses, "sig = ?";
        push @sql_param, $cfg->{signature};
    }
    if (@set_clauses) {
        my $set_clause = join(", ", @set_clauses);
        my $sql = "UPDATE users SET $set_clause WHERE uid = ?";
        push @sql_param, $uid;
        my $rs = $dbh->do($sql, undef, @sql_param);
        if (!$rs) {
            $self->disconnect_db;
            return;
        }
    }

    $self->disconnect_db;
    return $cfg;
}

1;
