package Newslash::Model::Users::Prefs;
use Newslash::Model::Base -base;

sub select {
    my $self = shift;
    my $params = {@_};

    my $uid = $params->{id} || $params->{uid} || $params->{user_id};
    return if !$uid;

    my $rs = $self->generic_select(table => "users_prefs",
                                   unique_keys => ["uid"],
                                   params => {uid => $uid});

    return $rs;
}

sub to_config {
    my ($self, $uid) = @_;
    my $new_cfg = { info => {},
                    comment => {},
                    ui => {},
                  };

    # from users table
    my $user = $self->generic_select(table => "users",
                                     unique_keys => ["uid"],
                                     params => {uid => $uid});
    if (!$user) {
        $self->warn("select invalid user; ID: $uid");
        return;
    }

    $new_cfg->{info}->{homepage} = defined $user->{homepage} ? $user->{homepage} : "";
    $new_cfg->{info}->{signature} = defined $user->{sig} ? $user->{sig} : "";

    # from user_comments table
    my $mapping = {
                   nosigs => 'hide_signature',
                   noscores => 'hide_score',
                  };
    my $comments = $self->generic_select(table => "users_comments",
                                         unique_keys => ["uid"],
                                         params => {uid => $uid});
    for my $k (keys %$mapping) {
        if (defined $comments->{$k} && $comments->{$k} =~ m/\A\d+\z/) {
            $new_cfg->{comment}->{$mapping->{$k}} = $comments->{$k} + 0;
        }
        else {
            $new_cfg->{comment}->{$mapping->{$k}} = $comments->{$k};
        }
    }

    # from user_info table
    my $info = $self->generic_select(table => "users_info",
                                     unique_keys => ["uid"],
                                     params => {uid => $uid});
    $info ||= {};
    $new_cfg->{info}->{realname} = defined $info->{realname} ? $info->{realname} : "";
    $new_cfg->{info}->{biography} = defined $info->{bio} ? $info->{bio} : "";

    # from user_param table
    my $old = $self->generic_select(table => "users_param",
                                     unique_keys => ["uid"],
                                    params => {uid => $uid});
    $old ||= {};
    $new_cfg->{comment}->{fold_threshold} = defined $old->{d2_highlightthresh} ? 0 + $old->{d2_highlightthresh} : 1;
    $new_cfg->{comment}->{show_threshold} = defined $old->{d2_threshold} ? 0 + $old->{d2_threshold} : 0;
    $new_cfg->{comment}->{auto_fold} = defined $old->{d2_reverse_switch} ? 0 + $old->{d2_reverse_switch} : 0;
    $new_cfg->{comment}->{notify_threshold} = defined $old->{message_threshold} ? 0 + $old->{message_threshold} : 0;

    if (defined $old->{domaintags}) {
        $new_cfg->{comment}->{show_domain} = $old->{domaintags} ? "auto" : "hide";
    }
    else {
        $new_cfg->{comment}->{show_domain} = "show";
    }
    if (defined $old->{d2_keybindings_switch}) {
        $new_cfg->{ui}->{enable_hotkey} = $old->{d2_keybindings_switch} ? 0 : 1;
    }
    else {
        $new_cfg->{ui}->{enable_hotkey} = 1;
    }

    # from users_prefs table
    my $prefs = $self->select(id => $user->{uid});
    $prefs ||= {};

    if ($prefs->{tzcode}) {
        $new_cfg->{ui}->{timezone} = $prefs->{tzcode};
    }
    if ($prefs->{mylinks}) {
        $new_cfg->{ui}->{my_links} = $prefs->{mylinks};
    }

    return $new_cfg;
}

sub convert_old_prefs {
    my ($self, $uid) = @_;
    my $new_cfg = $self->to_config($uid);
    my $users = $self->new_instance_of("Users");

    # apply configs
    my $cfg = {};
    my $cfg = $users->configs->select(id => $uid);

    return if !$cfg;

    my $updated = 0;
    for my $sect (keys %$new_cfg) {
        $cfg->{$sect} //= {};
        for my $k (keys %{$new_cfg->{$sect}}) {
            if (!defined $cfg->{$sect}->{$k}) {
                $cfg->{$sect}->{$k} = $new_cfg->{$sect}->{$k};
                $updated = 1;
            }
        }
    }

    # sidebar settings
    my $sidebar_items = $users->sidebar->select(uid => $uid);
    if (@$sidebar_items == 0) {
        # no sidebar items configured, try to convert
        my $ret = $users->index->select(uid => $uid);
        if ($ret && $ret->{slashboxes}) {
            my @slashboxes = split(/,/, $ret->{slashboxes});
            my @boxes;
            my $boxes = $self->new_instance_of("Boxes");
            for my $box_name (@slashboxes) {
                my $box = $boxes->select(name => $box_name);
                if (!$box) {
                    $self->warn("no box item: $box_name");
                    next;
                }
                push @boxes, $box->{id};
            }
            my $rs = $users->sidebar->update(uid => $uid, config => \@boxes);
            if (!$rs) {
                $self->warn("convert slashboxes failed: " . $self->last_error);
            }
        }
    }

    # add some default values
    if ($new_cfg->{ui}->{timezone}) {
        my $offset_sec = Newslash::Util::Timezones::get_timezone_offset($new_cfg->{ui}->{timezone});
        if (!defined $cfg->{ui}->{offset_sec} || $cfg->{ui}->{offset_sec} != $offset_sec) {
            $cfg->{ui}->{offset_sec} = $offset_sec;
            $updated = 1;
        }
    }

    if (!defined $cfg->{ui}->{time_format} || length($cfg->{ui}->{time_format}) ) {
        $cfg->{ui}->{time_format} = '%Y年%m月%d日 %H時%M分';
        $updated = 1;
    }

    if ($updated) {
        #my $rs = $users->configs->update(id => $uid, config => $cfg);
        #if (!$rs) {
        #    warn "update config failed. uid: $uid";
        #}
        return $cfg;
    }
    return;
}

1;
