package Newslash::Plugin::AntiCsrf;
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::Util qw(md5_sum dumper b64_encode);
use Mojo::Log;

use Crypt::OpenSSL::Random qw(random_seed random_bytes );

use constant TOKEN_SIZE => 64;

# set random seed
if (!random_seed(time())) {
    die 'random seed is not sufficient';
}

use constant KVS_KEY_PREFIX => 'csrf_tokens:';

sub _generate_token {
    my $self = shift;
    my $randoms = random_bytes(TOKEN_SIZE);
    if ($randoms) {
        my $token = b64_encode($randoms);
        $token =~ s/\s//g;
        return $token;
    }
    return;
}

sub register {
    my ($self, $app, $conf) = @_;

    $self->{app} = $app;
    $self->{kvs} = $app->kvs;
    $self->{expire} = 300; # expire is 300sec = 5min

    # set default config value
    my $cnf = $app->config->{AntiCsrf} ||= {};
    $cnf->{disable} = 0 if !defined $cnf->{disable};

    return if $cnf->{disable};

    if (!$self->{app}) {
        Mojo::Log->new->error('AntiCsrf: no app given');
    }
    if (!$self->{kvs}) {
        Mojo::Log->new->error('AntiCsrf: no kvs given');
    }

    $app->helper(anti_csrf => sub { state $anti_csrf = $self; });
    $app->hook(before_render => sub {
                   my ($c, $args) = @_;
                   return if !defined $args->{json};
                   my $check_id = $c->stash('csrf_check_id');
                   return if !$check_id;

                   my $session = $c->session('session');
                   my $token = $self->get_token($check_id, $session->{token});
                   $args->{json}->{csrf_token} = $token;
               });

    $app->hook(around_action => sub {
                   my ($next, $c, $action, $last) = @_;
                   return $next->() if $c->req->method ne 'POST';
                   my $check_id = $c->stash('csrf_check_id');
                   return $next->() if !$check_id;
                   my $token = "";
                   if ($c->req->json) {
                       my $action = $c->req->json->{action} || "";
                       return $next->() if $action eq 'preview';
                       $token = $c->req->json->{sec_token} || $c->req->json->{csrf_token};
                   }
                   else {
                       my $action = $c->req->param("action") || "";
                       return $next->() if $action eq 'preview';
                       $token = $c->req->param("sec_token") || $c->req->param("csrf_token");
                   }

                   my $session = $c->session('session');
                   if (!$token || !$self->validate_token($token, $check_id, $session->{token})) {
                       my $url = $c->url_for();
                       $token ||= "";
                       $c->app->log->debug("invalid csrf_token: $token for: $url");
                       # validation error
                       $c->render(json => { error => 1, reason => "invalid_csrf_token", message => "AntiCsrf: validation error" });
                       $c->rendered(400);
                       return;
                   }
                   return $next->();
               });

    # add 'csrf_token' helper
    $app->helper('csrf_token', sub {
                     my ($c, $rc_str) = @_;
                     my $session = $c->session('session');
                     # TODO: non-login user don't have a session token
                     my $token = $self->get_token($rc_str, $session->{token});
                     if ($token) {
                         return $token;
                     }
                     return;
                 });
}

sub get_token {
    my ($self, $resource_str, $session_token) = @_;
    return if !$resource_str;

    $session_token ||= "";
    my $check_str = "$resource_str:$session_token";

    my $token = $self->_generate_token;
    my $kvs_key = KVS_KEY_PREFIX . $token;
    my $existence = $self->{kvs}->get($kvs_key);
    while ($existence) {
        $token = $self->_generate_token;
        $existence = $self->{kvs}->get($kvs_key,);
    }
    my $rs = $self->{kvs}->set($kvs_key, $check_str, $self->{expire});
    if (!$rs) {
        $self->{app}->log->warn("AntiCsrf: cannot set token");
        return;
    }
    return $token;
}

sub validate_token {
    my ($self, $token, $resource_str, $session_token) = @_;
    return if !$token;
    return if !$resource_str;

    $session_token ||= "";
    my $check_str = "$resource_str:$session_token";

    my $kvs_key = KVS_KEY_PREFIX . $token;
    my $saved = $self->{kvs}->get($kvs_key);
    if ($saved && $saved eq $check_str) {
        return 1;
    }
    return;
}

1;
