package Newslash::Plugin::CompositeCache;
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::Log;

sub kvs { return shift->{kvs}; }
sub inmemory { return shift->{inmemory}; }
sub app { return shift->{app}; }

sub register {
    my ($self, $app, $conf) = @_;

    # merge global config
    my $cc_conf = $app->config->{Cache} ||= {};

    # define default cache expiration terms (sec.)
    $cc_conf->{term_units} ||= {};
    $cc_conf->{term_units}->{very_short} = 30;
    $cc_conf->{term_units}->{short}      = 180; # 3 minutes
    $cc_conf->{term_units}->{middle}     = 600; # 10 minutes
    $cc_conf->{term_units}->{long}       = 3600; # 60 minutes
    $cc_conf->{term_units}->{very_long}  = 43200; # 12 * 60 minutes;

    $cc_conf->{inmemory_ratio} ||= 0.5;

    # define expiration terms
    $cc_conf->{expire_terms} ||= {
                               "model:users"       => "short",
                               "model:stories"     => "very_short",
                               "model:journals"    => "very_short",
                               "model:comments"    => "very_short",
                               "model:timeline"    => "very_short",
                               "model:submissions" => "very_short",
                               "model:polls"       => "middle",
                               "model:ad_codes"    => "short",
                               "siteconfig"        => "long",
                               "user-boxes"        => "middle",
                               "JavaScriptLoader"  => "middle",
                               "messages"          => "short",
                               "anonymous_user"    => "middle",
                               "Preprocessor"      => "middle",
                              };
    if ($conf) {
        $cc_conf = { %$cc_conf, %$conf};
    }
    $self->{config} = $cc_conf;

    $self->{app} = $app;
    $self->{kvs} = $app->kvs;
    $self->{inmemory} = $app->cache;

    if (!$self->{app}) {
        Mojo::Log->new->warn('CompositeCache: no app given');
    }
    if (!$self->{kvs}) {
        Mojo::Log->new->warn('CompositeCache: no kvs given');
    }

    $app->helper(ccache => sub { state $ccache = $self });
}

sub _get_kvs_expire_term {
    my ($self, $base_key) = @_;
    my $unit = $self->{config}->{expire_terms}->{$base_key};
    if (!$unit) {
        $self->app->log->debug("CompositeCache: expiration term not given for key '$base_key'");
        $unit = "very_short";
    }

    return $self->{config}->{term_units}->{$unit} || $self->{config}->{term_units}->{very_short};
}

sub _get_expire_terms {
    my ($self, $base_key) = @_;
    my $base_term = $self->_get_kvs_expire_term($base_key);
    return (int($base_term * $self->{config}->{inmemory_ratio}), $base_term);
}


sub get {
    my ($self, $base_key, $variable_key, @rest) = @_;
    my $options = {@rest};

    if (!$base_key) {
        $self->app->log->error("CompositeCache::get: no base_key given.");
        return;
    }
    if (!$variable_key) {
        $self->app->log->error("CompositeCache::get: no variable_key given. base_key is '$base_key'");
        return;
    }

    if (!$self->inmemory && !$self->kvs) {
        # cache system is not available
        $self->app->log->warn("CompositeCache::get: no cache system available");
        return;
    }

    my ($mem_expire, $kvs_expire) = $self->_get_expire_terms($base_key);
    my $key = $base_key . ":" . $variable_key;

    # at first, check in-memory cache
    if ($self->inmemory) {
        my $result = $self->inmemory->get($key);
        if (defined $result) {
            $self->app->log->debug("CompositeCache::get: use inmemory cache for $key");
            return $result;
        }
    }
    else {
        $self->app->log->warn("CompositeCache::get: inmemory cache not available");
    }

    # next, check KVS cache
    if ($self->kvs) {
        my $result;
        if ($options->{mode} && $options->{mode} eq "hash") {
            $result = $self->kvs->hash_get($key);
        }
        else {
            $result = $self->kvs->get($key);
        }
        if (defined $result) {
            $self->app->log->debug("CompositeCache::get: use kvs cache for $key");

            # update in-memory cache
            if ($self->inmemory) {
                my $ttl = $self->kvs->ttl($key);
                if ($ttl > 0) {
                    $self->inmemory->set($key, $result, $ttl);
                }
            }
            return $result;
        }
    }
    else {
        $self->app->log->warn("CompositeCache::get: KVS cache not available");
        return;
    }

    $self->app->log->debug("CompositeCache::get: no cache found for $key");
    return;
}

sub hash_get {
    my ($self, $base_key, $variable_key) = @_;
    return $self->get($base_key, $variable_key, mode => "hash");
}


sub set {
    my ($self, $base_key, $variable_key, $value, @rest) = @_;
    my $options = {@rest};

    if (!$base_key) {
        $self->app->log->error("CompositeCache::cache: no base_key given.");
        return;
    }
    if (!$variable_key) {
        $self->app->log->error("CompositeCache::cache: no variable_key given.");
        return;
    }

    if (!$self->inmemory && !$self->kvs) {
        # cache system is not available
        $self->app->log->warn("CompositeCache::cache: no cache system available");
        return;
    }

    my ($mem_expire, $kvs_expire) = $self->_get_expire_terms($base_key);
    my $key = $base_key . ":" . $variable_key;

    # update in-memory cache
    if ($self->inmemory) {
        $self->inmemory->set($key, $value, $mem_expire);
    }
    else {
        $self->app->log->warn("CompositeCache::cache: inmemory cache not available");
    }

    # update KVS cache
    if ($self->kvs) {
        if ($options->{mode} && $options->{mode} eq "hash") {
            $self->kvs->hash_set($key, $value, $kvs_expire);
        }
        else {
            $self->kvs->set($key, $value, $kvs_expire);
        }
    }
    else {
        $self->app->log->warn("CompositeCache::cache: KVS cache not available");
    }
    return 1;
}

sub hash_set {
    my ($self, $base_key, $variable_key, $value) = @_;
    return $self->set($base_key, $variable_key, $value, mode => "hash");
}

sub del {
    my ($self, $base_key, $variable_key, $target) = @_;
    $target ||= "all";

    if ($target ne "all" && $target ne "inmemory" && $target ne "kvs") {
        $self->app->log->error("CompositeCache::del: invalid taget $target. $base_key - $variable_key");
        return;
    }

    if (!$base_key) {
        $self->app->log->error("CompositeCache::del: no base_key given.");
        return;
    }
    if (!$variable_key) {
        $self->app->log->error("CompositeCache::del: no variable_key given.");
        return;
    }

    if (!$self->inmemory && !$self->kvs) {
        # cache system is not available
        $self->app->log->warn("CompositeCache::del: no cache system available");
        return;
    }

    my $key = $base_key . ":" . $variable_key;

    # expire inmemory cache
    if ($self->inmemory && ($target eq "all" || $target eq "inmemory")) {
        my $rs = $self->inmemory->del($key);
        if (!$rs) {
            $self->app->log->error("CompositeCache::del: immemory cache delete failed for $base_key : $variable_key");
        }
    }

    # expire KVS cache
    if ($self->kvs && ($target eq "all" || $target eq "kvs")) {
        my $rs = $self->kvs->del($key);
        if (!$rs) {
            $self->app->log->error("CompositeCache::del: KVS cache delete failed for $base_key : $variable_key");
        }
    }
    return 1;
}

sub model {
    my ($self, $model_name, $options) = @_;
    $options ||= {};
    my $model = $self->{app}->model($model_name);
    if (!$model) {
        $self->app->log->warn("CompositeCache::model: invalid model name ($model_name)");
        return;
    }
    return Newslash::Plugin::CompositeCache::ModelWrapper->new($model,
                                                               $model_name,
                                                               $self->{app},
                                                               $options);
}

package Newslash::Plugin::CompositeCache::ModelWrapper;
use Mojo::Util qw(md5_sum dumper);
use Storable;
use Data::Dumper;

sub new {
    my ($class, $model, $model_name, $app, $options) = @_;
    if (!$model || !$model_name || !$app) {
        warn "CompositeCache::ModelWrapper::new: invalid arguments";
        return;
    }

    bless { _model => $model,
            _app => $app,
            _model_name => $model_name,
            options => $options || {},
          }, $class;
}

sub model_name { return shift->{_model_name}; }
sub model { return shift->{_model}; }
sub app { return shift->{_app}; }

sub _param_to_hash {
    my $self = shift;
    my @params = @_;

    my $serialized = Storable::freeze([@params]);
    return md5_sum($serialized);
}

sub _last_query {
    return shift->model->_last_query(@_);
}

sub select_nocache {
    my ($self, @params) = @_;
    return $self->model->select(@params);
}

sub select {
    my ($self, @params) = @_;

    if ($self->{options}->{no_cache}) {
        return $self->model->select(@params);
    }

    # generate key to access cached data
    my $base_key = "model:" . $self->model_name;
    my $variable_key = $self->_param_to_hash(@params);

    my $value = $self->app->ccache->get($base_key, $variable_key);
    if (!defined $value) {
        $value = $self->model->select(@params);
        if (defined $value) {
            $self->app->ccache->set($base_key, $variable_key, $value);
        }
    }
    return $value;
}

1;
