package Newslash::Plugin::KeyValueStore;
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::JSON qw(decode_json encode_json);

use Data::Dumper;
use Redis;

sub start_transaction {
    my $self = shift;
    if (!$self->{_multi}) {
        my $r = $self->_connect;
        $self->{_r}->multi;
        $self->{_multi} = 1;
    }
    return 1;
}

sub commit {
    my $self = shift;
    if (!$self->{_multi}) {
        return;
    }
    $self->{_r}->exec;
    $self->{_multi} = 0;
    $self->_quit;
    return 1;
}

sub rollback {
    my $self = shift;
    if (!$self->{_multi}) {
        return;
    }
    $self->{_r}->discard;
    $self->{_multi} = 0;
    $self->_quit;
    return 1;
}

sub _encode {
    my ($self, $item, $is_scalar) = @_;
    if ($is_scalar) {
        return $item;
    }
    else {
        return encode_json($item);
    }
}

sub _decode {
    my ($self, $item, $is_scalar) = @_;
    if ($is_scalar) {
        return $item;
    }
    else {
        return decode_json($item);
    }
}

sub _connect {
    my $self = shift;
    if ($self->{_r}) {
        return $self->{_r};
    }
    my $r;
    if ($self->{conf}->{sentinels}) {
	$r = Redis->new(sentinels => $self->{conf}->{sentinels},
			service => $self->{conf}->{service});
    }
    else {
	$r = Redis->new(server => $self->{conf}->{host});
    }
    $r->connect;
    $self->{_r} = $r;
    if (defined $self->{index_number}) {
        $r->select($self->{index_number});
    }
    return $r;
}

sub _quit {
    my $self = shift;
    return if !$self->{conf}->{always_disconnect};

    if ($self->{_multi}) {
        return;
    }
    if ($self->{_r}) {
        $self->{_r}->quit;
        delete $self->{_r};
    }
}

sub cache {
    my $self = shift;
    my $key = shift;
    my $expire = shift;
    my $func_ref = shift;
    my @args = @_;

    my $value = $self->get($key);

    if ($value) {
        # cache hit!
        return $value;
    }

    # cache unhit, then execute function and cache result
    $value = $func_ref->(@args);
    $self->set($key, $value, $expire);
    return $value;
}

sub time {
    my $self = shift;

    my $r = $self->_connect;
    my $result = $r->time;
    $self->_quit;
    return $result;
}

sub hkeys {
    my ($self, $key) = @_;

    my $r = $self->_connect;
    my $result = $r->hkeys($key);
    $self->_quit;
    return $result;
}

sub hexists {
    my ($self, $key, $field) = @_;

    my $r = $self->_connect;
    my $result = $r->hexists($key, $field);
    $self->_quit;
    return $result;
}

sub hincrby {
    my ($self, $key, $field, $increment) = @_;

    my $r = $self->_connect;
    my $result = $r->hincrby($key, $field, $increment);
    $self->_quit;
    return $result;
}

sub hset {
    my ($self, $key, $field, $value, @rest) = @_;

    my $options = {@rest};
    my $r = $self->_connect;
    my $result;
    if ($options->{scalar}) {
        $result = $r->hset($key, $field, $value);
    }
    else {
        $result = $r->hset($key, $field, $self->_encode($value));
    }
    $self->_quit;
    return $result;
}

sub hget {
    my ($self, $key, $field, @rest) = @_;
    my $options = {@rest};

    my $r = $self->_connect;
    my $result = $r->hget($key, $field);
    $self->_quit;
    if ($result) {
        if ($options->{scalar}) {
            return $result;
        }
        else {
            return $self->_decode($result);
        }
    } else {
        return;
    }
}

sub hdel {
    my $self = shift;
    my $key = shift;
    my $r = $self->_connect;
    my $result = $r->hdel($key, @_);
    $self->_quit;
    return $result;
}

sub atomic_hdel {
    my ($self, $key, $field, $value) = @_;
    my $r = $self->_connect;
    my $script = "if redis.call('HGET', KEYS[1], KEYS[2]) == ARGV[1] then return redis.call('HDEL', KEYS[1], KEYS[2]) else return 0 end";
    my $result = $r->eval($script, 2, $key, $field, $value);
    $self->_quit;
    return $result;
}

sub hash_set {
    my ($self, $key, $hash, $expire) = @_;
    if (ref($hash) ne "HASH") {
        return;
    }

    my $r = $self->_connect;
    my $result;

    my @args;
    push @args, $key;
    for my $item (%$hash) {
        if (ref($item)) {
            push @args, $self->_encode($item);
        }
        else {
            push @args, $item;
        }
    }
    $result = $r->hmset(@args);

    if (!$result) {
        $self->_quit;
        return $result;
    }
    if ($expire) {
        $r->expire($key, $expire);
    }
    $self->_quit;
    return $result;
}

sub hash_get {
    my ($self, $key) = @_;

    my $r = $self->_connect;
    my $result;
    if ($r->exists($key)) {
        $result = $r->hgetall($key);
    }
    $self->_quit;

    if ($result) {
        return {@$result};
    }
    return;
}

sub set {
    my ($self, $key, $value, $expire, $flag, @rest) = @_;
    my $option = {@rest};
    my $r = $self->_connect;
    my $result;
    my @args;
    push @args, $key;
    push @args, $self->_encode($value, $option->{scalar});
    if ($expire) {
        push @args, 'EX';
        push @args, $expire;
    }
    if ($flag) {
        push @args, $flag;
    }
    #$result = $r->set($key, $self->_encode($value), );
    $result = $r->set(@args);
    $self->_quit;
    return $result;
}

sub get {
    my $self = shift;
    my $key = shift;

    my $r = $self->_connect;
    my $result = $r->get($key);
    $self->_quit;
    if ($result) {
        return $self->_decode($result);
    } else {
        return;
    }
}

sub del {
    my ($self, $key) = @_;
    my $r = $self->_connect;
    my $result = $r->del($key);
    $self->_quit;
    return $result;
}

sub ttl {
    my ($self, $key) = @_;
    my $r = $self->_connect;
    my $result = $r->ttl($key);
    $self->_quit;
    return $result;
}

sub expire {
    my ($self, $key, $expire_sec) = @_;
    my $r = $self->_connect;
    my $result = $r->expire($key, $expire_sec);
    $self->_quit;
    return $result;
}

sub register {
    my ($self, $app, $conf) = @_;

    my $cnf = $app->config->{KeyValueStore} ||= {};
    $cnf->{type} ||= "redis";
    $cnf->{host} ||= "localhost:6379";
    $cnf->{index_number} ||= 0;

    $self->{conf} = {%$conf, %$cnf};
    $self->{index_number} = $self->{conf}->{index_number} || 0;

    $self->{app} = $app;
    $app->helper(kvs => sub { state $kvs = $self; });
}

1;
