package Newslash::Plugin::Redirect;
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::Util qw(md5_sum dumper);
use List::Util qw(any);
use Socket;

=encoding utf8

=head1 NAME

Newslash::Plugin::Redirect - redirect control plugin for Newslash

=head1 SYNOPSIS

  # Mojolicious
  $app->plugin('Newslash::Plugin::Redirect');

=head1 DESCRIPTION

L<Newslash::Plugin::Redirect> controls redirection.


=head1 METHODS

=head2 register

  $plugin->register(Mojolicious->new);

Register hooks in L<Mojolicious> application.

=cut

sub register {
    my ($self, $app, $conf) = @_;

    my @sections = qw(apple askslashdot azure developers
		      hardware idle interview it linux
		      mainpage mobile opensource review
		      science security slash srad yro);
    my $sections_rex = join("|", @sections);


    my $rules = [
		 [ qr/^\/(?:m|i)\/$/ => '/' ],
		 [ qr/^\/(?:\w+\.)?srad\.jp\/(.*)$/ => '/$1/' ],
		 [ qr/^\/\/(?:\w+\.)?srad\.jp\/(.*)$/ => '/$1/' ],
		 [ qr/^\/(?:\w+\/)?([0-9\/]+)\.shtml$/ => '/story/$1/' ],
		 [ qr/^\/($sections_rex)\/$/ => '/tag/$1/' ],
		 [ qr/^\/faq\/(\w+).shtml$/ => '/faq/$1/' ],
		 [ qr/^\/hof\.\w+$/ => '/hof/' ],
		 [ qr/^\/supporters\.\w+$/ => '/supporters/' ],
		 [ qr/^\/stories\/?$/ => '/' ],
		 [ qr/^\/stories\/(\w+)\/?$/ => '/tag/$1/' ],
		 [ qr/^\/privacy(?:\.\w+)$/ => 'https://osdn.jp/privacy.shtml' ],
		 [ qr/^\/polls\/(\d+)$/ => '/poll/$1/' ],
		];


    my $query_rules = [
		       [ qr/^\/(?:\w+\/)?article\.pl$/ => ["sid"] =>  '/story/$sid/' ],
		       [ qr/^\/(?:\w+\/)?comments\.pl$/ => ["cid"] =>  '/comment/$cid/' ],
		      ];

    $app->hook(around_dispatch => sub {
		 my ($next, $c) = @_;

		 # check subdomain redirection
		 my $hostname = ($c->config->{Site} || {})->{hostname} || "";
		 if ($hostname) {
		     my $request_host = $c->req->url->to_abs->host_port;
		     if ($request_host ne $hostname) {
			 my $subdomain = $request_host;
			 $subdomain =~ s/\.$hostname$//;

			 if ($subdomain) {
			     my $new_url = $c->req->url;
			     $new_url->host($hostname);
			     $c->res->code(301);
			     $c->redirect_to($new_url);
			     return;
			 }
		     }
		 }

		 # path redirection
		 my $url = $c->req->url;
		 for my $rule (@$rules) {
		     my $path = $url->path;
		     if ($path =~ $rule->[0]) {
			 my $m = $1;
			 $path = $rule->[1];
			 $path =~ s/\$1/$m/;
			 $url->path($path);
			 $c->res->code(301);
			 $c->redirect_to($url);
			 return;
		     }
		 }

		 # query redirection
		 $url = $c->req->url;
		 for my $rule (@$query_rules) {
		     my $path = $url->path;
		     my $cnt = 0;
		     if ($path =~ $rule->[0]) {
			 $path = $rule->[2];
			 for my $q (@{$rule->[1]}) {
			     my $val = $url->query->param($q);
			     if ($val) {
				 $cnt++;
				 $path =~ s/\$$q/$val/;
			     }
			 }
			 if ($cnt == @{$rule->[1]}) {
			     $url->path($path);
			     $url->query('');
			     $c->res->code(301);
			     $c->redirect_to($url);
			 }
		     }
		 }

		 $next->();
	       });
}


1;
