package Newslash::Plugin::UserAuth;
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::Util qw(md5_sum b64_encode dumper);
use Mojo::JSON qw(to_json);
use List::Util qw(any);

use Socket;
use Crypt::OpenSSL::Random qw(random_seed random_bytes );

use constant KVS_KEY_PREFIX => 'sessions:';
use constant KVS_KEY_PREFIX_REV => 'users:session:';

sub _generate_token {
    my $self = shift;
    my $randoms = random_bytes(128);
    if ($randoms) {
        return b64_encode($randoms);
    }
    return undef;
}

sub register {
    my ($self, $app, $conf) = @_;

    my $cnf = $app->config->{UserAuth} ||= {};
    $cnf->{bypass} ||= ['^/favicon.ico$', "^/img/", "^/css/", "^/js/"];

    # set random seed
    if (!random_seed(time())) {
        die 'random seed is not sufficient';
    }

    $app->hook(before_dispatch => sub {
                   my $c = shift;

                   my $path = $c->req->url->to_abs->path;
                   if (any { $path =~ m/$_/ } @{$cnf->{bypass}}) {
                       $c->app->log->debug("UserAuth: bypass user authentification for $path");
                       return;
                   }

                   my $session = $c->session('session');
                   my $remote_ip = $c->tx->remote_address;

                   my $user;
                   if ($session->{token}) {
                       $user = $c->kvs->get(KVS_KEY_PREFIX . $session->{token});
                   }
                   if (!$user) {
                       $user = $self->_get_anonymous_user;
                   }
                   $user->{ipid} = ip_id($remote_ip);
                   $user->{subnetid} = subnet_id($remote_ip);
                   $user->{srcids} = {
                                      24 => srcid($remote_ip, 24),
                                      32 => srcid($remote_ip, 32),
                                     };

                   # delete private informations
                   delete $user->{passwd};
                   delete $user->{newpasswd};
                   delete $user->{newpasswd_ts};

                   if ($user->{is_login}) {
                       my $value = $c->ccache->get("messages", "$user->{uid}");
                       if (!defined $value) {
                           $value = $c->model('web_messages')->count_unread($user->{uid});
                       }
                       $user->{messages} = $value;
                   }

                   $c->stash(user => $user);
               });

    $self->{app} = $app;
    $self->{kvs} = $app->kvs;

    if (!$self->{app}) {
        Mojo::Log->new->warn('UserAuth: no app given');
    }
    if (!$self->{kvs}) {
        Mojo::Log->new->warn('UserAuth: no kvs given');
    }
    $app->helper(user_auth => sub { state $user_auth = $self; });
}

sub _add_user_properties {
    my ($self, $user) = @_;
    return if !$user;
    my $users = $self->{app}->model('users');
    $users->get_user_config($user);

    # add permissions, configs
    $user->{permissions} = $users->permissions->get(user => $user) || {};

    my $converted = $users->prefs->convert_old_prefs($user->{uid});
    if ($converted) {
        $user->{config} = $converted;
        $user->{configJSON} = to_json($converted);
    }

    if ($user->{is_login}) {
        # set user relations
        $user->{relations} = $self->{app}->model('relations')->hash($user->{uid});
    }
}

sub _add_user_configs {
    my ($self, $user) = @_;
    return if !$user;
}

sub do_login {
    my ($self, $user, $c) = @_;
    return if (!$user || !$c);
    my $users = $c->model('users');
    $users->get_user_config($user);

    # convert old preferences
    my $converted = $users->prefs->convert_old_prefs($user->{uid});
    if ($converted) {
        # if convert executed, update conf
        my $rs = $users->configs->update(id => $user->{uid}, config => $converted);
        if (!$rs) {
            $self->app->log->error("update config failed. uid: $user->{uid}");
        }
        $user->{config} = $converted;
        $user->{configJSON} = to_json($converted);
    }
    $self->_add_user_properties($user);

    my $token = $c->user_auth->store_session($user);
    return if !$token;

    my $session = {
                   token => $token,
                   timestamp => time(),
                   remote_addr => $c->req->env->{REMOTE_ADDR},
                  };
    $c->session(session => $session);

    # get session expiration
    my $conf = $c->app->config('System') || {};
    $c->session(expiration => $conf->{session_expiration} || 3600);
    return 1;
}

sub _get_anonymous_user {
    my $self = shift;
    my $params = {@_};
    my $user;

    if (!$params->{nocache}) {
        $user = $self->{app}->ccache->get('anonymous_user', 1);
    }
    if (!$user) {
        my $users = $self->{app}->model('users');
        $user = $users->anonymous_user;
        $self->_add_user_properties($user);
        my $expire = 300; # expire time is 300 sec (5 min)
        $self->{app}->ccache->set('anonymous_user', 1, $user);
    }

    return $user;
}

sub update_user_cache {
    my ($self, $user) = @_;
    if ($user->{is_login}) {
        $user = $self->{app}->model('users')->select(uid => $user->{uid});
        $self->_add_user_properties($user);
        return $self->update_session($user);
    }
    else {
        # Anonymous User
        $user = $self->_get_anonymous_user(nocache => 1);
    }
    return 1;
}


sub store_session {
    my ($self, $user) = @_;

    my $token = $self->_generate_token;
    my $kvs_key = KVS_KEY_PREFIX . $token;

    # check token's overlap, if so, regenerate
    my $existence = $self->{kvs}->get($kvs_key);
    while ($existence) {
        $token = $self->_generate_token;
        $existence = $self->{kvs}->get($kvs_key);
    }

    my $rs1 = $self->{kvs}->set($kvs_key, $user);
    my $rs2 = $self->{kvs}->set(KVS_KEY_PREFIX_REV . $user->{matchname}, $kvs_key);
    if ($rs1 && $rs2) {
        return $token;
    }
    return;
}

sub update_session {
    my ($self, $user, $session) = @_;
    my $kvs_key = "";
    if (!$session) {
        $kvs_key = $self->{kvs}->get(KVS_KEY_PREFIX_REV . $user->{matchname});
    }
    else {
        $kvs_key = KVS_KEY_PREFIX . $session->{token};
    }
    return if !$kvs_key;
    my $rs = $self->{kvs}->set($kvs_key, $user);
    return $rs;
}

sub clear_session {
    my ($self, $user, $token) = @_;
    my $kvs_key = KVS_KEY_PREFIX . $token;
    $self->{kvs}->del($kvs_key);
    if ($user && $user->{matchname}) {
        $self->{kvs}->del(KVS_KEY_PREFIX_REV . $user->{matchname});
    }
}

# from slash's get_ipids method (Slash::Utility::Environment)
sub ip_id {
    my $remote_ip = shift;
    return $remote_ip ? md5_sum($remote_ip) : '';
}

sub subnet_id {
    my $subnet_id = shift;
    $subnet_id =~ s/(\d+\.\d+\.\d+)\.\d+/$1\.0/;
    return $subnet_id ? md5_sum($subnet_id): '';
}

sub srcid {
    my ($ip, $mask) = @_;

    my $prepend_code = sprintf "%02x", 0x20 + (32 - $mask);
    my $n = unpack "N", inet_aton($ip);
    my $bitmask = ~( 2**(32-$mask) - 1 );
    my $nm = $n & $bitmask;

    my $str = inet_ntoa(pack("N", $nm));
    my $md5 = md5_sum($str);
    my $md5_trunc = substr $md5, 0, 14;

    return lc "$prepend_code$md5_trunc";
}

1;
