package Newslash::Plugin::WikiContentsReader;
use Mojo::Base 'Mojolicious::Plugin';

use LWP::UserAgent;
use HTTP::Request;
use HTTP::Headers;
use HTML::Filter;
use XML::RSS;
use File::Path;
use File::Basename;
use HTTP::Date;

has "app";

=encoding utf8

=head1 NAME

Newslash::Plugin::WikiContentsReader - Read Wiki Content

=head1 SYNOPSIS

  # Mojolicious
  $app->plugin('Newslash::Plugin::WikiContentsReaders');

=head1 DESCRIPTION

L<Newslash::Plugin::WikiContentsReaders> is a helper plugin to
read wiki content from osdn.jp.


=head1 METHODS

=head2 register

  $plugin->register(Mojolicious->new);

Register helpers in L<Mojolicious> application.

=cut

sub register {
    my ($self, $app, $conf) = @_;
    $self->app($app);

    $app->helper(wiki_reader => sub { state $wiki_reader = $self });
}

=head1 HELPERS

L<Mojolicious::Plugin::WikiContentsReaders> implements the following helpers.

=cut

sub _read {
    my ($self, $page_name) = @_;

    my $cf = $self->app->config;
    my $config = $cf->{WikiContents} ||= {};
    my $con_timeout = $config->{connect_timeout} //= 10;
    my $req_timeout = $config->{request_timeout} //= 10;
    my $max_redirects = $config->{max_redirects} //= 10;

    my $http_proxy = $config->{http_proxy};
    my $https_proxy = $config->{https_proxy};

    my $project = $config->{project_name} //= "";
    my $wiki_url = "http://osdn.jp/projects/$project/wiki";
    my $wiki_url_regex = "https?://[0-9a-zA-Z.]*osdn.(jp|net)/projects/$project/wiki/";
    my $page_url = "$wiki_url/$page_name";

    if (!$project) {
        $self->app->log->error("WikiContentsReader::_read: no project given.");
        return;
    }

    # get content
    my $ua  = Mojo::UserAgent->new;
    $ua->connect_timeout($con_timeout);
    $ua->request_timeout($req_timeout);
    $ua->max_redirects($max_redirects);
    $ua->proxy->http($http_proxy) if $http_proxy;
    $ua->proxy->https($https_proxy) if $https_proxy;


    my $tx = $ua->get($page_url);
    if (!$tx) {
        $self->app->log->error("WikiContentsReader::_read: get '$page_name' failed.");
        return;
    }
    if ($tx->res->code != 200) {
        my $code = $tx->res->code;
        $self->app->log->error("WikiContentsReader::_read: get '$page_name' failed. code: $code");
        return;
    }

    my $html = Encode::decode_utf8($tx->res->body);

    # replace anchor URL
    my $p = Newslash::Plugin::WikiContentsReader::Filter->new;
    $p->set_replace_url($wiki_url_regex, '/');
    $p->parse($html);

    my $content = { body => $p->filtered_html,
                    title => $p->{title},
                    page_name => $page_name,
                  };

    if (!$content->{title} && $html =~ m!<title>([^<]+)</title>!si) {
        $content->{title} = $1;
    }

    $content->{body} =~ s/\A.*(<div class=\"wiki-content\">.*)<!-- google_ad_section_end -->.*\z/$1/s;
    if (!$content->{body}) {
        return;
    }

    $content->{body} =~ s/<script type='text\/javascript'>.*?<\/script>//sg;
    if (!$content->{body}) {
        return;
    }

    return $content;

}

=head2 read($page_name)

  get content from srad wiki (https://osdn.net/projects/slashdotjp/wiki/)

=cut

sub get {
    my ($self, $page_name) = @_;
    my $cache_key = "wiki_page";
    my $content = $self->app->ccache->get($cache_key, $page_name);

    if (defined $content) {
        return $content;
    }

    $content = $self->_read($page_name);
    return if !$content;

    $self->app->ccache->set($cache_key, $page_name, $content);
    return $content;
}



# filter for anchor URL replace
package Newslash::Plugin::WikiContentsReader::Filter;
use Mojo::Base 'HTML::Filter';

sub set_replace_url {
    my ($self, $from, $to) = @_;
    $self->{url_from} = $from;
    $self->{url_to} = $to;
}

sub get_title {
    return shift->{title};
}

sub start {
    my ($self, $tagname, $attrs, $attrnames, $text) = @_;

    if ($tagname eq "a" && $self->{url_from} && $self->{url_to} && $attrs->{href}) {
        if ($attrs->{href} =~ s/^$self->{url_from}//) {
            $attrs->{href} =~ s!-!/!;
            $attrs->{href} = "$self->{url_to}$attrs->{href}";
            $attrs->{href} =~ s!^/+!/!;
            $attrs->{href} =~ s!^/faq$!/faq/!;
        }
        if ($attrs->{href} !~ m!/attach/!) {
            $text = "<$tagname " . join(" ", map { "$_=\"$attrs->{$_}\"" } @$attrnames) . ">";
        }
    } elsif ($tagname eq "form") {
        $self->{form_seen}++;
    } elsif ($tagname eq "h1") {
        $self->{h1_seen}++;
    }

    $self->output($text);
}

sub end {
    my $self = shift @_;
    my ($tagname, $text) = @_;

    if ($tagname eq "form") {
        $self->{form_seen}--;
    } elsif ($tagname eq "h1") {
        $self->{h1_seen}++;
    }

    $self->output($text);
}

sub text {
    my $self = shift @_;
    my ($text) = @_;

    if ($self->{h1_seen} && int($self->{h1_seen}) == 3) {
        $self->{title} = $text;
        $self->{h1_seen}++;
    }

    $self->output($text);
}

sub output { push(@{$_[0]->{fhtml}}, $_[1]) unless ($_[0]->{form_seen}); }
sub filtered_html { join("", @{$_[0]->{fhtml}}) }

1;

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.

=cut
