package Newslash::Util::TestMan;

use strict;
use warnings;
use utf8;
use feature ':5.10';

use Exporter 'import';
use Newslash::Util::Test;
use constant REMOTE_IP => "192.168.1.1";
use Newslash::Plugin::UserAuth;
use Newslash::Util::TestMan::Mail;

use Test::More;

sub new {
    my ($class, $test, $options) = @_;
    $options ||= {};
    if (!$test) {
        warn "$test not given!";
        return;
    }

    bless {
           options => $options,
           users => [],
           stories => [],
           comments => [],
           submissions => [],
           test => $test,
           mail_que => [],
          }, $class;
}

sub app {
    return shift->{test}->app;
}

sub t {
    return shift->{test};
}

sub mail_que {
    return shift->{mail_que};
}

sub srcid {
    my ($self, $ip, $mask) = @_;
    return Newslash::Plugin::UserAuth::srcid($ip, $mask);
}

sub create_user {
    my ($self, $name, $passwd) = @_;
    my $user = Newslash::Util::Test::create_user($self->app, $name, $passwd);
    if (!$user) {
        fail("create test user - $name / $passwd");
        return;
    }
    $user->{srcids} = {
                       24 => Newslash::Util::Test::srcid(REMOTE_IP, 24),
                       32 => Newslash::Util::Test::srcid(REMOTE_IP, 32),
                      };
    $user->{_passwd} = $passwd;

    push @{$self->{users}}, $user;
    return $user;
}

sub login {
    my ($self, $user, $passwd) = @_;
    $passwd //= $user->{_passwd};
    $self->t->post_ok('/my/login'
                      => {Accept => '*/*'}
                      => form
                      => {nickname => $user->{nickname}, passwd => $passwd})
      ->status_is(302);
}

sub logout {
    my ($self) = @_;
    $self->t->get_ok('/my/logout')->status_is(302);
}

sub create_admin {
    my ($self, $name, $passwd) = @_;
    my $user = Newslash::Util::Test::create_admin_user($self->app, $name, $passwd);
    if (!$user) {
        warn "admin create failed!";
        return;
    }
    $user->{srcids} = {
                       24 => Newslash::Util::Test::srcid(REMOTE_IP, 24),
                       32 => Newslash::Util::Test::srcid(REMOTE_IP, 32),
                      };
    $user->{_passwd} = $passwd;

    push @{$self->{users}}, $user;
    return $user;
}

sub create_comment {
    my $self = shift;
    my $params = {@_};
    for my $k (qw(user title comment)) {
        return if !$params->{$k};
    }
    my $discussion_id = $params->{discussion} || $params->{discussion_id};
    if (!$discussion_id) {
        return;
    }
    my $comment = Newslash::Util::Test::create_comment($self->app,
                                                       $params->{user},
                                                       $discussion_id,
                                                       $params->{title},
                                                       $params->{comment},
                                                       $params->{pid});
    push @{$self->{comments}}, $comment;
    return $comment;
}

sub create_story {
    my $self = shift;
    my $params = {@_};
    my $story = Newslash::Util::Test::create_story($self->app, $params->{user},
                                                   params => $params);
    push @{$self->{stories}}, $story;
    return $story;
}

sub create_journal {
    my $self = shift;
    my $params = {@_};

    my $j = Newslash::Util::Test::create_journal($self->app,
                                                 $params->{user},
                                                 params => $params);
    push @{$self->{journals}}, $j;
    return $j;
}

sub create_submission {
    my $self = shift;
    my $params = {@_};

    my $user;
    if ($params->{user}) {
	$user = $params->{user};
    }
    else {
	$user = Newslash::Util::Test::get_anonymous_user($self->app);
    }

    my $s = Newslash::Util::Test::create_submission($self->app,
                                                    $user,
                                                    params => $params);
    ok($s, "create submission for test (Newslash::Util::TestMan)");
    push @{$self->{submissions}}, $s;
    return $s;
}

sub process_event {
    my $self = shift;
    my $params = {@_};
    my $type = $params->{event_type};
    my $subtype = $params->{event_subtype};
    my $src_id = $params->{source_user_id};

    if (!$type || !$subtype || !$src_id) {
        fail("TestMan::process_event failed: invalid parameter");
        return;
    }
    my $events = $self->app->model('events');
    my $ev = $events->select(event_type => $type,
                             event_subtype => $subtype,
                             source_user_id => $src_id,
                             active => 1,
                            );
    if(!$ev || @$ev == 0) {
        fail("TestMan::process_event failed: cannot retrieve event - $type/$subtype $src_id");
        return;
    }

    # check if event processed correctly
    my $handlers = Newslash::Eventd::Handlers->new($self->app, { test => 1 });
    my $rs = $handlers->execute($ev->[0]);
    if(!$rs) {
        fail("TestMan::process_event failed: event processing failed");
        return;
    }

    # check email que
    my $user_handler = $handlers->{_handlers}->{$type};
    if(!$user_handler) {
        fail("TestMan::process_event failed: cannot retrieve '$type' event handler");
        return;
    }
    $self->{mail_que} = $user_handler->{_test_mailque};
    return 1;
}

sub pop_mailque {
    my ($self, $message) = @_;
    $message ||= "TestMan::pop_mailque";
    my $mail = pop @{$self->mail_que} || {};
    ok($mail, $message);
    return Newslash::Util::TestMan::Mail->new($mail);
}

sub delete_user {
    my ($self, $user) = @_;
    Newslash::Util::Test::delete_user($self->app, $user->{nickname});
}


sub cleanup {
    my $self = shift;
    my $item;

    for $item (@{$self->{users}}) {
        Newslash::Util::Test::delete_user($self->app, $item->{nickname});
    }
    for $item (@{$self->{comments}}) {
        Newslash::Util::Test::delete_comment($self->app, $item->{cid});
    }
    for my $item (@{$self->{stories}}) {
        Newslash::Util::Test::delete_story($self->app, $item->{stoid});
    }
    for my $item (@{$self->{journals}}) {
        Newslash::Util::Test::delete_journal($self->app, $item->{jornal_id});
    }
    for my $item (@{$self->{submissions}}) {
        Newslash::Util::Test::delete_submission($self->app, $item->{submission_id});
    }
}

1;


=encoding utf8

=head1 NAME

Newslash::Util::TestMan - Newslash Test Helper

=head1 SYNOPSIS

  use Newslash::Util::TestMan;
  my $t = Test::Mojo->new('Newslash::Web');
  my $test_man = Newslash::Util::TestMan->new($t);

  if ($t->app->mode eq 'test') {
      my $user = $test_man->create_user("nickname", "password");
      my $admin = $test_man->create_admin("admin_nickname", "password");

      $test_man->login($user);
      $test_man->logout;
  }
  $test_man->cleanup;

=head1 DESCRIPTION

L<Newslash::Util::TestMan> is helper object to test L<Newslash>.

=head1 METHODS


=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.

=cut
