# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2009 by Geeknet, Inc. See README
# and COPYING for more information, or see http://slashcode.com/.

package Newslash::Util::TextFormatter;

use strict;
use warnings;
use utf8;
use feature ':5.10';

use Exporter 'import';
our @EXPORT_OK = qw(strip_by_mode);

use EscapeHTML qw();
use HTML::Tidy;
use Data::Dumper;
use Encode;

use constant ANCHOR    => -4;
use constant NOTAGS    => -3;
use constant ATTRIBUTE => -2;
use constant LITERAL   => -1;
use constant NOHTML    => 0;
use constant PLAINTEXT => 1;
use constant HTML      => 2;
use constant EXTRANS   => 3;
use constant CODE      => 4;

use constant APPROVED_URL_SCHEMES_REGEX => qr/(?-xism:(?:ftp|http|gopher|mailto|news|nntp|telnet|wais|https))/;

#========================================================================

=head2 breakHtml(TEXT, MAX_WORD_LENGTH)

Private function.  Break up long words in some text.  Will ignore the
contents of HTML tags.  Called from C<stripByMode> functions -- if
there are any HTML tags in the text, C<stripBadHtml> will have been
called first.  Handles spaces before dot-words so as to best work around a
Microsoft bug.  This code largely contributed by Joe Groff <joe at pknet
dot com>.

=over 4

=item Parameters

=over 4

=item TEXT

The text to be fixed.

=item MAX_WORD_LENGTH

The maximum length of a word.  Default is 50 (breakhtml_wordlength in vars).

=back

=item Return value

The text.

=back

=cut

use constant BREAK_HTML_WORDLENGTH => 8000;
use constant COMMENT_STARTWORD_WORKAROUND => 0;
use constant TWEAK_JAPANESE => 1;
use constant APPROVEDTAGS_BREAK => ['p','br','ol','ul','li','dl','dt',
                                    'dd','blockquote','div','img','hr',
                                    'h1','h2','h3','h4','h5','h6'
                                   ];
use constant COMMENT_NONSTARTWORDCHARS_REGEX =>
  qr{(?i-xsm:(\s+)((?:<[^>]+>)*(?:[\.\,\;\:\\/]|&#(?:46|x2e|44|x2c|59|x3b|58|x3a|47|x2f);)+)(\S))};
use constant APPROVEDTAGS => [ 'b','i','p','br','a','ol','ul',
                               'li','dl','dt','dd','em','strong',
                               'tt','blockquote','div','ecode','del',
                               'ins','sub','sup','quote','strike'
                             ];
use constant APPROVEDTAGS_ATTR => {
                                  'ol' => { 'start' => {'ord' => 1} },
                                  'a' => {
                                          'href' => { 'req' => 1,
                                                      'ord' => 1,
                                                      'url' => 1,
                                                    }
                                         },
                                  'blockquote' => { 'title' => {'ord' => 2},
                                                    'cite' => {'ord' => 1,'url' => 1}
                                                  },
                                  'img' => { 'width' => {'ord' => 3},
                                             'alt' => {'req' => 2,'ord' => 2},
                                             'longdesc' => {'ord' => 5,'url' => 1},
                                             'src' => {'req' => 1,'ord' => 1,'url' => 1},
                                             'height' => {'ord' => 4},
                                           },
                                 };

sub approve_tag {
    my ($whole_tag, $opts) = shift;
    my $allowed_tags = $opts->{allowed_tags} || {};
    my $r = EscapeHTML::escape_tag($allowed_tags, $whole_tag);
    return "<$r>";
};

sub approve_charref {
    my $char_ref = shift;
    return $char_ref;
}

sub url_to_html {
    my $text = shift;
    return '' if !defined $text || $text eq '';

    my $scheme_regex = APPROVED_URL_SCHEMES_REGEX;

    # we know this can break real URLs, but probably will
    # preserve real URLs more often than it will break them
    # was ['":=>]
    # should we parse the HTML instead?  problematic ...
    $text =~  s{(?<!\S)((?:$scheme_regex):/{0,2}[$URI::uric#]+)}{
        #my $url   = fudgeurl($1);
        my $url = $1;
        my $extra = '';
        $extra = $1 if $url =~ s/([?!;:.,']+)$//;
        $extra = ')' . $extra if $url !~ /\(/ && $url =~ s/\)$//;
        #print STDERR "url2html s/// url='$url' extra='$extra'\n" if !defined($url) || !defined($extra);
        qq[<a href="$url" rel="url2html-$$">$url</a>$extra];
    }ogie;
    # url2html-$$ is so we can remove the whole thing later for ecode

    return $text;
}

sub break_html {
    my($text, $mwl) = @_;
    $mwl = $mwl || BREAK_HTML_WORDLENGTH;

    # Only do the <NOBR> and <WBR> bug workaround if wanted.
    my $workaround_start = COMMENT_STARTWORD_WORKAROUND
      ? "<nobr>" : "";
    my $workaround_end = COMMENT_STARTWORD_WORKAROUND
      ? "<wbr></nobr> " : " ";

    if (TWEAK_JAPANESE) {
        $workaround_start = "";
        $workaround_end = "";
    }

    # These are tags that "break" a word;
    # a<P>b</P> breaks words, y<B>z</B> does not
    my $approvedtags_break = APPROVEDTAGS_BREAK || [];
    my $break_tag = join '|', @$approvedtags_break;
    $break_tag = qr{(?:$break_tag)}i;

    # This is the regex that finds a char that, at the start of
    # a word, will trigger Microsoft's bug.  It's already been
    # set up for us, it just needs a shorter name.
    my $nswcr = COMMENT_NONSTARTWORDCHARS_REGEX;

    # And we also need a regex that will find an HTML entity or
    # character references, excluding ones that would break words:
    # a non-breaking entity.  For now, let's assume *all* entities
    # are non-breaking (except an encoded space which would be
    # kinda dumb).
    my $nbe = qr{ (?:
                      &
                      (?! \# (?:32|x20) )
                      (\#?[a-zA-Z0-9]+)
                      ;
                  ) }xi;

    # Mark off breaking tags, as we don't want them counted as
    # part of long words
    $text =~ s{
                  (</?$break_tag>)
          }{\x00$1\x00}gsx;

    # Temporarily hide whitespace inside tags so that the regex below
    # won't accidentally catch attributes, e.g. the HREF= of an A tag.
    # (Which I don't think it can do anyway, based on the way the
    # following regex gobbles <> and the fact that tags should already
    # be balanced by this point...but this can't hurt - Jamie)
    1 while $text =~ s{
                          (<[^>\s]*)	# Seek in a tab up to its
                          \s+		# first whitespace
                  }{$1\x00}gsx;		# and replace the space with NUL

    # Put the <wbr> in front of attempts to exploit MSIE's
    # half-braindead adherance to Unicode char breaking.
    $text =~ s{$nswcr}{<nobr> <wbr></nobr>$2$3}gs if COMMENT_STARTWORD_WORKAROUND;

    # Break up overlong words, treating entities/character references
    # as single characters and ignoring HTML tags.
    $text =~ s{(
                   (?:^|\G|\s)		# Must start at a word bound
                   (?:
                       (?>(?:<[^>]+>)*)	# Eat up HTML tags
                       (			# followed by either
                           $nbe		# an entity (char. ref.)
                       |	(?!$nbe)\S	# or an ordinary char
                       )
                   ){$mwl}			# $mwl non-HTML-tag chars in a row
               )}{
                   substr($1, 0, -length($2))
                     . $workaround_start
                     . substr($1, -length($2))
                     . $workaround_end
                 }gsex;

    # Just to be tidy, if we appended that word break at the very end
    # of the text, eliminate it.
    $text =~ s{<nobr> <wbr></nobr>\s*$}{} if COMMENT_STARTWORD_WORKAROUND;

    # Fix breaking tags
    $text =~ s{
                  \x00
                  (</?$break_tag>)
                  \x00
          }{$1}gsx;

    # Change other NULs back to whitespace.
    $text =~ s{\x00}{ }g;

    return $text;
}


#========================================================================

=head2 processCustomTagsPre(STRING)

=head2 processCustomTagsPost(STRING)

Private function.  It does processing of special custom tags (in Pre, ECODE;
in Post, QUOTE).

=over 4

=item Parameters

=over 4

=item STRING

String to be processed.

=back

=item Return value

Processed string.

=item Dependencies

Pre is meant to be used before C<approveTag> is called; Post after.
Both are called only from regular posting modes, HTML and PLAINTEXT.

=back

=cut

sub process_custom_tags_pre {
    my ($str) = @_;

	## Deal with special ECODE tag (Embedded Code).  This tag allows
	## embedding the Code postmode in plain or HTML modes.  It may be
	## of the form:
	##    <ECODE>literal text</ECODE>
	## or, for the case where "</ECODE>" is needed in the text:
	##    <ECODE END="SOMETAG">literal text</SOMETAG>
	##
	## SOMETAG must match /^\w+$/.
	##
	##
	## Note that we also strip out leading and trailing newlines
	## surrounding the tags, because in plain text mode this can
	## be hard to manage, so we manage it for the user.
	##
	## Also note that this won't work if the site disallows TT
	## or BLOCKQUOTE tags.
	##
	## -- pudge

    # ECODE must be in approvedtags
	if (grep /^ecode$/i, @{APPROVEDTAGS()}) {
		$str =~ s|<(/?)literal>|<${1}ecode>|gi;  # we used to accept "literal" too
		my $ecode   = 'ecode';
		my $open    = qr[\n* <\s* (?:$ecode) (?: \s+ END="(\w+)")? \s*> \n*]xsio;
		my $close_1 = qr[($open (.*?) \n* <\s* /\2    \s*> \n*)]xsio;  # if END is used
		my $close_2 = qr[($open (.*?) \n* <\s* /ECODE \s*> \n*)]xsio;  # if END is not used

		while ($str =~ m[($open)]g) {
			my $len = length($1);
			my $end = $2;
			my $pos = pos($str) - $len;

			my $close = $end ? $close_1 : $close_2;
			my $substr = substr($str, $pos);
			if ($substr =~ m/^$close/si) {
				my $len = length($1);
				my $codestr = $3;
				# remove these if they were added by url2html; I know
				# this is a rather cheesy way to do this, but c'est la vie
				# -- pudge
				$codestr =~ s{<a href="[^"]+" rel="url2html-$$">(.+?)</a>}{$1}g;
				my $code = strip_code($codestr);
				my $newstr = "<p><blockquote>$code</blockquote></p>";
				substr($str, $pos, $len) = $newstr;
				pos($str) = $pos + length($newstr);
			}
		}
	}
	return $str;
}

sub process_custom_tags_post {
	my($str) = @_;

	# QUOTE must be in approvedtags
	if (grep /^quote$/i, @{APPROVEDTAGS()}) {
		my $quote   = 'quote';
		my $open    = qr[\n* <\s*  $quote \s*> \n*]xsio;
		my $close   = qr[\n* <\s* /$quote \s*> \n*]xsio;

		$str =~ s/$open/<p><div class="quote">/g;
		$str =~ s/$close/<\/div><\/p>/g;
	}

	# just fix the whitespace for blockquote to something that looks
	# universally good
	if (grep /^blockquote$/i, @{APPROVEDTAGS()}) {
		my $quote   = 'blockquote';
		my $open    = qr[\s* <\s*  $quote \s*> \n*]xsio;
		my $close   = qr[\s* <\s* /$quote \s*> \n*]xsio;

		$str =~ s/(?<!<p>)$open/<p><$quote>/g;
	}

	return $str;
}



my %action_data = ();

my %actions = (
	newline_to_local => sub {
			${$_[0]} =~ s/(?:\015?\012|\015)/\n/g;		},
	trailing_whitespace => sub {
			${$_[0]} =~ s/[\t ]+\n/\n/g;			},
	encode_html_amp => sub {
			${$_[0]} =~ s/&/&amp;/g;			},
	encode_html_amp_ifnotent => sub {
			${$_[0]} =~ s/&(?!#?[a-zA-Z0-9]+;)/&amp;/g;	},
	encode_html_ltgt => sub {
			${$_[0]} =~ s/</&lt;/g;
			${$_[0]} =~ s/>/&gt;/g;				},
	encode_html_ltgt_stray => sub {
			1 while ${$_[0]} =~ s{
				( (?: ^ | > ) [^<]* )
				>
			}{$1&gt;}gx;
			1 while ${$_[0]} =~ s{
				<
				( [^>]* (?: < | $ ) )
				>
			}{&lt;$1}gx;					},
	encode_html_quote => sub {
			${$_[0]} =~ s/"/&#34;/g;			},
	breakHtml_ifwhitefix => sub {
			${$_[0]} = break_html(${$_[0]})
				unless $action_data{no_white_fix};	},
	processCustomTagsPre => sub {
			${$_[0]} = process_custom_tags_pre(${$_[0]});	},
	processCustomTagsPost => sub {
			${$_[0]} = process_custom_tags_post(${$_[0]});	},
	approveTags => sub {
			${$_[0]} =~ s/<(.*?)>/approve_tag($1, $_[2])/sge;	},
	url2html => sub {
			${$_[0]} = url_to_html(${$_[0]});			},
	approveCharrefs => sub {
			${$_[0]} =~ s{
				&(\#?[a-zA-Z0-9]+);?
			}{approve_charref($1)}gex;			},
	space_between_tags => sub {
			${$_[0]} =~ s/></> </g;				},
	whitespace_tagify => sub {
			${$_[0]} =~ s/\n/<br>/gi;  # pp breaks
			${$_[0]} =~ s/(?:<br>\s*){2,}<br>/<br><br>/gi;
			# Preserve leading indents / spaces
			# can mess up internal tabs, oh well
			${$_[0]} =~ s/\t/    /g;			},
	paragraph_wrap => sub {
			# start off the text with a <p>!
			${$_[0]} = '<p>' . ${$_[0]} unless ${$_[0]} =~ /^\s*<p>/s;
			# this doesn't assume there will be only two BRs,
			# but it does come after whitespace_tagify, so
			# chances are, will be only two BRs in a row
			${$_[0]} =~ s/(?:<br>){2}/<p>/g;
			# make sure we don't end with a <br><p> or <br>
			${$_[0]} =~ s/<br>(<p>|$)/$1/g;			},
	whitespace_and_tt => sub {
			${$_[0]} =~ s{((?:  )+)(?: (\S))?} {
				("&nbsp; " x (length($1)/2)) .
				(defined($2) ? "&nbsp;$2" : "")
			}eg;
			${$_[0]} = "<tt>${$_[0]}</tt>";			},
	newline_indent => sub {
			${$_[0]} =~ s{<br>\n?( +)} {
				"<br>\n" . ('&nbsp; ' x length($1))
			}ieg;						},
	remove_tags => sub {
			${$_[0]} =~ s/<.*?>//gs;			},
	remove_ltgt => sub {
			${$_[0]} =~ s/<//g;
			${$_[0]} =~ s/>//g;				},
	remove_trailing_lts => sub {
			${$_[0]} =~ s/<(?!.*?>)//gs;			},
	remove_newlines => sub {
			${$_[0]} =~ s/\n+//g;				},

);

my %mode_actions = (
	PLAINTEXT, [qw(
			newline_to_local
			trailing_whitespace
			processCustomTagsPre
			remove_trailing_lts
			approveTags
			processCustomTagsPost
			space_between_tags
			encode_html_ltgt_stray
			encode_html_amp_ifnotent
			approveCharrefs
			breakHtml_ifwhitefix
			whitespace_tagify
			newline_indent
			paragraph_wrap			)],
	HTML, [qw(
			newline_to_local
			trailing_whitespace
			processCustomTagsPre
			remove_trailing_lts
			approveTags
			processCustomTagsPost
			space_between_tags
			encode_html_ltgt_stray
			encode_html_amp_ifnotent
			approveCharrefs
			breakHtml_ifwhitefix		)],
	CODE, [qw(
			newline_to_local
			trailing_whitespace
			encode_html_amp
			encode_html_ltgt
			whitespace_tagify
			whitespace_and_tt
			breakHtml_ifwhitefix		)],
	EXTRANS, [qw(
			newline_to_local
			trailing_whitespace
			encode_html_amp
			encode_html_ltgt
			breakHtml_ifwhitefix
			whitespace_tagify
			newline_indent			)],
);

sub strip_by_mode {
    my ($str, $fmode, @rest) = @_;
    my $opts = {@rest};
    return "" if !defined $str;
    return if !defined $fmode;

    $action_data{no_white_fix} = $opts->{no_white_fix} || 0;

    my @actions = @{$mode_actions{$fmode}};
    for my $action (@actions) {
        $actions{$action}->(\$str, $fmode, $opts);
    }
    return $str;
}

sub strip_mode {
	my($string, $mode, @args) = @_;
	return "" if !$mode || $mode < 1 || $mode > 4;	# user-supplied modes are 1-4
	return strip_by_mode($string, $mode, @args);
}

sub strip_code {
    return strip_by_mode(shift, CODE);
}

sub contextual_strip {
    my ($text, $max_chars, $min_chars) = @_;
    return if !$text;
    return if !$max_chars;
    $min_chars ||= 0;

    # split by "。"
    my @tmp = split(/(。)/, $text);
    my @sentences;
    my $prev;
    for my $s (@tmp) {
        if ($s eq "。") {
            $prev = $prev . $s;
        } else {
            push @sentences, $prev if $prev;
            $prev = $s;
        }
    }
    push @sentences, $prev;

    my $result = "";
    if (@sentences == 0) {
        $result = $text;
    }
    elsif (@sentences == 1) {
        $result = $sentences[0];
    }
    else {
        for my $sentence (@sentences) {
            my $tmp = $result . $sentence;
            if (length($tmp) > $max_chars) {
                last if (length($tmp) > $min_chars);
                $result = $tmp;
                last;
            }
            $result = $tmp;
        }
    }
    if (length($result) > $max_chars) {
        $result = substr($result, 0, $max_chars);
        $result = $result . " ...";
    }

    return $result;
}

=head1 FUNCTIONS

=head2 tidy_html($html)

  tidy HTML, then returns result.

=cut

sub tidy_html {
    my ($html) = @_;
    my $tidy = HTML::Tidy->new( {
                                 tidy_mark => 0,
                                 wrap => 0,
                                 doctype => 'omit',
                                 'show-body-only' => 1,
                                } );
    my $tidyed = $tidy->clean($html);
    chomp($tidyed);
    return $tidyed;
}

=head2 escape_html(\%allowed, $html)

  escape HTML, then returns result.

=cut

sub escape_html {
    my ($allowed, $html) = @_;
    return EscapeHTML::escape($allowed, $html);
}

=head2 blank_line_to_paragra($html)

  add <P> tag to paragraph, then returns result.

=cut

sub blank_line_to_paragraph {
    my ($html,) = @_;
    return EscapeHTML::blank_line_to_paragraph($html);
}

=head2 clean_html(\%allowed, $html)

  escape and tidy HTML, then returns result.

=cut

sub clean_html {
    my ($allowed, $html) = @_;
    my $escaped = escape_html($allowed, $html);
    $escaped = EscapeHTML::blank_line_to_paragraph($escaped);
    return tidy_html($escaped);
}

=head2 escape_plaintext($text)

  escape plaintext, then returns result.

=cut

sub escape_plaintext {
    my ($text) = @_;

    $text =~ s/(?!|lt;|gt;)/&amp;/g;
    $text =~ s/</&lt;/g;
    $text =~ s/>/&gt;/g;

    return $text;
}

1;

