package Newslash::Web;
use Mojo::Base 'Mojolicious';
use Mojo::Util qw(dumper);
use List::Util qw(any);

use constant CONFIG_FILE => '/etc/newslash/newslash.conf';
has subcommand => "";

# This method will run once at server start
sub startup {
    my $app = shift;

    if ($ARGV[0]) {
        $app->subcommand($ARGV[0]);
    }

    # add commands in Newslash::Command
    push @{$app->commands->namespaces}, 'Newslash::Command';

    # load config file
    # first, check existence of /etc/newslash.conf
    if ($app->mode eq 'production' && -e CONFIG_FILE) {
        $app->plugin('Newslash::Plugin::YAMLConfig', file => CONFIG_FILE);
    }
    else {
	if ($app->subcommand eq "configdump") {
	    $app->plugin('Newslash::Plugin::YAMLConfig', default => {});
	}
	else {
	    $app->plugin('Newslash::Plugin::YAMLConfig');
	}
    }

    # load default configuration values
    $app->plugin('Newslash::Plugin::DefaultConfig');

    # TODO: load/save configs with database

    if ($app->config->{Log} && $app->config->{Log}->{backtrace}) {
        require Carp::Always;
        Carp::Always->import;
    }

    # system log config
    if ($app->config->{Log} && $app->config->{Log}->{system_log}) {
	my $cnf = $app->config->{Log}->{system_log};

	# set log level
	if ($cnf->{level}) {
	    my $loglv = $cnf->{level};
	    if (grep { $loglv eq $_ } qw(debug info warn error fatal)) {
		$app->log->level($loglv);
		$app->log->info("set log level to $loglv");
	    }
	    else {
		$app->log->warn('invalid log level given in config file');
	    }
	}

	# file output settings
	if ($cnf->{mode} eq "local_file") {
	    my $pathname = $cnf->{local_file};
	    # check log is writable
	    if (!$pathname) {
		$app->log->error("cannot write system log to file: filename not given");
	    }
	    elsif (-e $pathname) {
		if (-w $pathname) {
		    $app->log->debug("logs will be outputed to $pathname ...");
		    $app->log(Mojo::Log->new);
		    $app->log->path($pathname);
		}
		else {
		    $app->log->error("cannot write system log to file: $pathname");
		}
	    }
	    else {
		if (open(my $fh, ">", $pathname)) {
		    close($fh);
		    $app->log->debug("logs will be outputed to $pathname ...");
		    $app->log(Mojo::Log->new);
		    $app->log->path($pathname);
		}
		else {
		    $app->log->error("cannot create system log file: $pathname");
		}
	    }
        }
    }

    # add static contents directories
    if ($app->config->{System} && $app->config->{System}->{static_dir}) {
        for my $dir (@{$app->config->{System}->{static_dir}}) {
            push @{$app->static->paths}, $dir;
        }
    }

    # secret key for hasing
    $app->secrets([$app->config->{System}->{secret_key},]);

    # when "test" mode, output debug logs.
    $app->log->level('debug') if $app->mode eq 'test';

    # check if 'maintenance' mode
    my $maintenance_mode = 0;
    if ($app->subcommand
        && any { $app->subcommand eq $_ } qw[configdump configimport databaseinit
                                             testdatainsert useradd usermod
                                           ]) {
	$maintenance_mode = 1;
    }

    ############################################################
    #
    # Plugin Settings
    #
    ############################################################

    # profiler settings
    if ($app->mode eq 'development'
        && $app->config->{Profiler}
        && $app->config->{Profiler}->{enable}) {
        if ($app->config->{Profiler}->{profiler} eq 'nytprof') {
            $app->plugin('Mojolicious::Plugin::NYTProf', { nytprof => {} });
        }
    }

    # enable access logging
    $app->plugin('Newslash::Plugin::AccessLog');

    # redirect handlers
    $app->plugin('Newslash::Plugin::Redirect');

    # Helpers for Newslash
    $app->plugin('Newslash::Plugin::NewslashHelpers');

    # use Epoch
    $app->plugin('Newslash::Plugin::Epoch');

    # support BasicAuth
    $app->plugin('Newslash::Plugin::BasicAuth');

    # use TimeLimitedCache ($app->cache)
    $app->plugin('Newslash::Plugin::TimeLimitedCache');

    # use KeyValue Store ($app->kvs)
    $app->plugin('Newslash::Plugin::KeyValueStore');

    # add Model Loader
    $app->plugin('Newslash::Plugin::Model', { bypass_startup => $maintenance_mode });

    # use CompositeCache Cache ($app->ccache)
    $app->plugin('Newslash::Plugin::CompositeCache');

    # use Template::Toolkit 2 render
    $app->plugin('Newslash::Plugin::TT2Renderer');

    # use ResponseFilter (Faculities)
    $app->plugin('Newslash::Plugin::ResponseFilter');

    # use CustomBoxes
    $app->plugin('Newslash::Plugin::CustomBoxes');

    # use AntiCsrf
    $app->plugin('Newslash::Plugin::AntiCsrf');

    # contents preprocessor
    $app->plugin('Newslash::Plugin::Preprocessor');
    if ($maintenance_mode) {
        $app->log->info("bypassing preprocessor...");
    }
    else {
	$app->preprocessor->generate_all;
    }

    # javascript loader
    $app->plugin('Newslash::Plugin::JavaScriptLoader');
    if ($maintenance_mode) {
        $app->log->info("bypassing javascriptloader...");
    }
    else {
	$app->javascript_loader->load_all_bundles;
    }

    # user authorization
    $app->plugin('Newslash::Plugin::UserAuth');

    # access control
    $app->plugin('Newslash::Plugin::AccessControl');

    # ReCaptcha control
    $app->plugin('Newslash::Plugin::ReCaptcha');

    # Post Filter
    $app->plugin('Newslash::Plugin::PostFilter');

    # DiscussionHelper
    $app->plugin('Newslash::Plugin::DiscussionHelper');

    # use HSTS
    $app->plugin('Newslash::Plugin::Hsts');

    # Event Que
    $app->plugin('Newslash::Plugin::EventQue');

    # Stats Logger
    $app->plugin('Newslash::Plugin::Stats');

    # Request Body based routing condition
    $app->plugin('Newslash::Plugin::RequestBodyCondition');

    # NS-RPC
    $app->plugin('Newslash::Plugin::NSRPC');

    # AD renderer
    $app->plugin('Newslash::Plugin::ADRenderer');

    # HTTP Compression
    $app->plugin('Newslash::Plugin::HttpCompression');

    # Users helper
    $app->plugin('Newslash::Plugin::Users');

    # Stories helper
    $app->plugin('Newslash::Plugin::Stories');

    # Sendmail helper
    $app->plugin('Newslash::Plugin::Sendmail');

    # Wiki contents reader helper
    $app->plugin('Newslash::Plugin::WikiContentsReader');

    # Set page's property automatically
    $app->plugin('Newslash::Plugin::AddPageProperty');


    ############################################################
    #
    # Routing Settings
    #
    ############################################################

    my $r = $app->routes;

    # index page
    $r->get('/')->to('timeline#top_page');
    $r->get('/recent')->to('timeline#recent');
    $r->get('/popular')->to('timeline#popular');
    $r->get('/comments')->to('timeline#comments');
    $r->get('/journals')->to('timeline#journals');
    $r->get('/submissions')->to('timeline#submissions');
    $r->get('/polls')->to('timeline#polls');

    # RSS
    $r->get('/:rss_type' => [format => ['rss', 'xml']])->to('rss#stories');
    $r->get('/journals/rss')->to('rss#journals');

    # Banned page
    $r->get('/banned')->to('index#banned', noindex => 1, no_ad => 1);

    # Login / Logout
    $r->get('/my/login')->to('login#login', no_ad => 1);
    $r->post('/my/login')->to('login#login');
    $r->get('/my/logout')->to('login#logout');
    $r->get('/login' => sub {my $c = shift; $c->res->code(301); $c->redirect_to('/my/login');});
    $r->get('/logout' => sub {my $c = shift; $c->res->code(301); $c->redirect_to('/my/logout');});

    # story page
    $r->get('/story/:sid/' => [sid => qr|\d\d/\d\d/\d\d/\d+|])
      ->to('story#single');

    # comment page
    $r->get('/comment/:cid/')->to('comment#single');

    # journal page
    # also see user page settings for /~username/journal/
    $r->get('/journal/new')->to('journal#create', seclev => 1, no_ad => 1, use_captcha => 1);
    $r->get('/journal/:id/')->to('journal#single');

    # submission page
    $r->get('/submission/new')->to('submission#create', no_ad => 1, use_captcha => 1);
    $r->get('/submission/:id/')->to('submission#single');
    #$r->post('/submission')->to('submission#create');

    # polls page
    $r->get('/poll/:qid')->to('poll#single');
    $r->get('/vote/:qid')->to('poll#vote');
    #$r->post('/vote/:qid')->to('poll#vote_post', csrf_check_id => 'vote');
    $r->post('/vote/:qid')->to('poll#vote_post');

    # archive page
    $r->get('/story/:year/:month/:day/')->to('archive#story');
    $r->get('/story/:year/:month/')->to('archive#story');

    # redirect rule for /stroy/<tags or topics>/
    $r->get('/story/:tag/' => [tag => qr{[^/ ]+}] => sub {
        my $c = shift;
        my $tag = $c->stash('tag');
        $c->res->code(301);
        $c->redirect_to("/tag/$tag/story/");
    });
    $r->get('/story/')->to('archive#story');

    $r->get('/journal/:year/:month/:day/')->to('archive#journal');
    $r->get('/journal/:year/:month/')->to('archive#journal');
    $r->get('/journal/')->to('archive#journal');

    $r->get('/submission/:year/:month/:day/')->to('archive#submission');
    $r->get('/submission/:year/:month/')->to('archive#submission');
    $r->get('/submission/')->to('archive#submission');

    $r->get('/comment/:year/:month/:day/')->to('archive#comment');
    $r->get('/comment/:year/:month/')->to('archive#comment');
    $r->get('/comment/')->to('archive#comment');

    $r->get('/poll/:year/:month/')->to('archive#poll');
    $r->get('/poll/')->to('archive#poll');

    # tag page
    $r->get('/tag/:tagname/:type/:page/')->to('tag#list_tagged_items');
    $r->get('/tag/:tagname/:page/' => [page => qr/\d+/])->to('tag#list_tagged_items');
    $r->get('/tag/:tagname/:type/')->to('tag#list_tagged_items');
    $r->get('/tag/:tagname/')->to('tag#list_tagged_items');
    $r->get('/tag/')->to('tag#list_tags');

    # my page
    $r->get('/my/settings')->to('my#settings', seclev => 1);
    $r->get('/my/sidebar')->to('my#sidebar', seclev => 1);
    $r->get('/my/inbox')->to('my#messages', seclev => 1);
    $r->get('/my/')->to('user#home', seclev => 1);

    # User Register
    $r->get('/my/newuser')->to('login#newuser', no_ad => 1);
    $r->post('/my/newuser')->to('login#newuser', captcha_check => 1);
    $r->get('/my/activation')->to('login#activation', no_ad => 1);
    $r->get('/my/resetpassword')->to('login#reset_password', no_ad => 1);
    $r->post('/my/resetpassword')->to('login#reset_password', captcha_check => 1);

    # Change Email
    $r->get('/my/change_email')->to('my#change_email', seclev => 1, captcha_check => 1);

    # search page
    $r->get('/search')->to('search#search');

    # firehose, index (for compatibility)
    $r->get('/firehose')->to('redirect#firehose');
    $r->get('/index2')->to('redirect#firehose');
    $r->get('/index')->to('redirect#index');
    $r->get('/users.pl')->to('redirect#users');
    $r->get('/:section/index')->to('redirect#index');

    # about
    $r->get('/about')->to('about#about');

    # Hall of Fames
    $r->get('/hof')->to('hof#hof');

    # faq
    $r->get('/faq/editorial')->to('faq#editorial');
    $r->get('/faq/UI')->to('faq#UI');
    $r->get('/faq/firehose')->to('faq#firehose');
    $r->get('/faq/com-mod')->to('faq#com_mod');
    $r->get('/faq/metamod')->to('faq#metamod');
    $r->get('/faq/accounts')->to('faq#accounts');
    $r->get('/faq/friends')->to('faq#friends');
    $r->get('/faq/tech')->to('faq#tech');
    $r->get('/faq/')->to('faq#faq');

    $r->get('/supporters')->to('faq#supporters');

    # topics
    $r->get('/topics.pl' => sub {my $c = shift; $c->res->code(301); $c->redirect_to('/topics/');});
    $r->get('/topics')->to('topics#topics');

    # authors
    $r->get('/authors')->to('authors#authors');

    # Admin
    # pages under /admin needs seclev equal or greater than 10000;
    my $admin = $r->under('/admin' => sub { my $c = shift; $c->stash(seclev => 10000); return 1; })->detour(no_ad => 1);

    $admin->get('/firehose/:id/')->to('admin-firehose#single');
    $admin->get('/submissions')->to('admin-submissions#index');

    $admin->get('/css')->to('admin-css#edit');
    $admin->get('/story/edit')->to('admin-story#edit');
    $admin->get('/story/list')->to('admin-story#list');

    $admin->get('/users')->to('admin-users#index');

    $admin->get('/default-sidebar')->to('admin-sidebar#defaults');

    $admin->get('/sidebar')->to('admin-sidebar#index');
    $admin->get('/feed')->to('admin-feed#index');
    $admin->get('/filter')->to('admin-filter#index');
    $admin->get('/blocking')->to('admin-blocking#index');
    $admin->get('/ad')->to('admin-ads#index');

    $admin->get('/repository')->to('admin-repository#index');
    $admin->get('/cache')->to('admin-config#cache');

    # Admin API
    # pages under /api/v1/admin needs seclev equal or greater than 10000;
    my $admin_api = $r->under('/api/v1/admin' => sub { my $c = shift; $c->stash(seclev => 10000); return 1; });

    $admin_api->get('/feed')->to('API::Admin::Feed#get');
    $admin_api->post('/feed')->to('API::Admin::Feed#post');
    $admin_api->get('/filter')->to('API::Admin::Filter#get');
    $admin_api->post('/filter')->to('API::Admin::Filter#post');
    $admin_api->get('/blocking')->to('API::Admin::Blocking#get');
    $admin_api->post('/blocking')->to('API::Admin::Blocking#post');

    $admin_api->get('/repository/export')->to('API::Admin::Repository#export');
    $admin_api->get('/repository/import')->to('API::Admin::Repository#import');

    $admin_api->post('/sidebar')->to('API::Admin::Sidebar#post');
    $admin_api->get('/sidebar')->to('API::Admin::Sidebar#get');

    $admin_api->get('/story')->to('API::Story#get');
    $admin_api->post('/story')->to('API::Story#post');

    # ad codes management
    $app->rpc->route_to_model($admin_api->get('/ad/code/'), 'ad_codes', 'select');
    $app->rpc->route_to_model($admin_api->post('/ad/code/')->over(request_body => {action => "create"}), 'ad_codes', 'create');
    $app->rpc->route_to_model($admin_api->post('/ad/code/')->over(request_body => {action => "update"}), 'ad_codes', 'update');
    $app->rpc->route_to_model($admin_api->post('/ad/code/')->over(request_body => {action => "delete"}), 'ad_codes', 'delete');


    # API
    my $api = $r->under('/api/v1');
    $api->post('/login')->to('API::Login#login');

    $api->post('/newuser/validate')->to('API::User#validate_new_user');
    $api->post('/newuser/create')->to('API::User#create_new_user');
    $api->post('/newuser/password')->to('API::User#newuser_password');

    $api->get('/sidebar/item')->to('API::SidebarItem#get', seclev => 1);

    $api->get('/comment')->to('API::Comment#get');
    $api->post('/comment')->to('API::Comment#post', captcha_check => 1, csrf_check_id => 'comment');

    $api->get('/user')->to('API::User#get');
    $api->post('/user')->to('API::User#post', seclev => 1);

    $api->get('/journal')->to('API::Journal#get');
    $api->post('/journal')->to('API::Journal#post', seclev => 1, csrf_check_id => 'journal');

    $api->get('/submission')->to('API::Submission#get');
    $api->get('/submissions')->to('API::Submission#list');
    $api->post('/submission')->to('API::Submission#post', captcha_check => 1, csrf_check_id => 'submission');

    $api->get('/timeline')->to('API::Timeline#get');

    $api->get('/poll')->to('API::Poll#get');
    $api->post('/poll')->to('API::Poll#post');
    $api->post('/vote')->to('API::Poll#vote', csrf_check_id => 'vote');

    $api->get('/moderation')->to('API::Moderation#get');
    $api->post('/moderation')->to('API::Moderation#post', seclev => 1, csrf_check_id => 'moderation');

    $api->get('/metamoderation')->to('API::Metamoderation#get');
    $api->post('/metamoderation')->to('API::Metamoderation#post', seclev => 1, csrf_check_id => 'moderation');

    $api->post('/relation')->to('API::Relation#post', seclev => 1, csrf_check_id => 'relation');

    $api->get('/token')->to('API::Token#get');

    $api->post('/tag')->to('API::Tag#post');
    $api->post('/tagmoderation')->to('API::TagModeration#post');

    # user page
    # warning: these pathes uses regexp matching, so must write in tail of route definitions.
    my $user = $r->under('/~#nickname');
    $user->get('/'             => [nickname => qr/.*/])->to('user#home');
    $user->get('/journal/:id'  => [nickname => qr/.*/])->to('journal#single');

    $user->get('/journal/:year/:month:day/'  => [nickname => qr/.*/])->to('archive#user_journal');
    $user->get('/journal/:year/:month/'  => [nickname => qr/.*/])->to('archive#user_journal');

    $user->get('/journals'     => [nickname => qr/.*/])->to('user#journals');
    $user->get('/journal'      => [nickname => qr/.*/])->to('user#journals'); # for compatibility
    $user->get('/comments'     => [nickname => qr/.*/])->to('user#comments');
    $user->get('/submissions'  => [nickname => qr/.*/])->to('user#submissions');
    $user->get('/friends'      => [nickname => qr/.*/])->to('user#friends');
    $user->get('/foes'         => [nickname => qr/.*/])->to('user#foes');
    $user->get('/fans'         => [nickname => qr/.*/])->to('user#fans');
    $user->get('/freaks'       => [nickname => qr/.*/])->to('user#freaks');
    $user->get('/achievements' => [nickname => qr/.*/])->to('user#achievements');

}

1;
