# -*- coding: utf-8 -*-
"""wrapper: Wrapper interface to Full-text search system 'lucene'"""

import lucene
from java.io import File
from org.apache.lucene.util import Version
from org.apache.lucene.index import (IndexWriter,
                                     IndexWriterConfig,
                                     IndexReader,
                                     Term,
                                     DirectoryReader)
                                     
from org.apache.lucene.search import (IndexSearcher,
                                      BooleanClause,
                                      TopScoreDocCollector,
                                      TopFieldCollector,
                                      TotalHitCountCollector,
                                      TermQuery as _TermQuery,
                                      NumericRangeQuery,
                                      BooleanQuery as _BooleanQuery,
                                      Sort as _Sort,
                                      SortField,
)
from org.apache.lucene.analysis.tokenattributes import CharTermAttribute

from org.apache.lucene.search.highlight import (SimpleHTMLFormatter,
                                                Highlighter as _Highlighter,
                                                QueryScorer,
                                                TokenSources,
)

from org.apache.lucene.analysis.ja import JapaneseAnalyzer, JapaneseTokenizer
from org.apache.lucene.store import SimpleFSDirectory
from org.apache.lucene.document import (Field as _Field, TextField, StringField, IntField,
                                        Document as LuceneDocument)
from org.apache.lucene.queryparser.flexible.standard import StandardQueryParser
from org.apache.lucene.queryparser.flexible.standard.config import StandardQueryConfigHandler


def init_vm():
    lucene.initVM()

class LuceneWrapperError(Exception):
    pass

class LuceneWrapper(object):
    def __init__(self, **kwargs):
        self._config = kwargs
        self._version = Version.LUCENE_4_10_1
        self._reader = None

    def _get_analyzer(self):
        # create Japanese analyzer
        mode = JapaneseTokenizer.Mode.NORMAL
        stop_set = JapaneseAnalyzer.getDefaultStopSet()
        stop_tags = JapaneseAnalyzer.getDefaultStopTags()
        return JapaneseAnalyzer(None, mode, stop_set, stop_tags)

    def _get_index_directory(self):
        return SimpleFSDirectory(File(self._config["index_directory"]))

    def _get_searcher(self):
        self._searcher = IndexSearcher(self._get_reader())
        return self._searcher

    def _get_reader(self):
        index_dir = self._get_index_directory()
        self._reader = DirectoryReader.open(index_dir)
        return self._reader

class Indexer(LuceneWrapper):
    def __init__(self, **kwargs):
        super(Indexer, self).__init__(**kwargs)
        self._writer = None
    
    def _get_writer(self):
        analyzer = self._get_analyzer()

        # create IndexWriterConfig
        lucene_conf = IndexWriterConfig(self._version, analyzer)

        # index directory
        index_dir = self._get_index_directory()

        # create IndexWriter
        self._writer = IndexWriter(index_dir, lucene_conf)
        return self

    def _close_writer(self):
        if self._writer:
            self._writer.close()
            self._writer = None
        return self

    def add(self, doc):
        self._writer.addDocument(doc.doc)
        return self

    def update(self, term, doc):
        self._writer.updateDocument(term, doc.doc)
        return self

    def delete(self, query):
        self._writer.deleteDocuments(query.query)

    def delete_all(self):
        self._writer.deleteAll()

    def __enter__(self):
        self._get_writer()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._writer.commit()
        self._close_writer()
    

class Document(LuceneWrapper):
    def __init__(self):
        super(Document, self).__init__()
        self.doc = LuceneDocument()

    def _store_flag(self, store):
        if store:
            return _Field.Store.YES
        return _Field.Store.NO

    def add_string_field(self, name, value, store=True):
        self.doc.add(StringField(name, value, self._store_flag(store)))
        return self

    def add_text_field(self, name, value, store=True):
        self.doc.add(TextField(name, value, self._store_flag(store)))
        return self

    def add_int_field(self, name, value, store=True):
        self.doc.add(IntField(name, value, self._store_flag(store)))
        return self

    def remove_field(self, name):
        self.doc.removeField(name)

    def remove_fields(self, name):
        self.doc.removeFields(name)


class Reader(LuceneWrapper):
    def __init__(self, **kwargs):
        super(Reader, self).__init__(**kwargs)
        self._get_reader()

    def get_term_vector(self, doc_id, field_name):
        return self._reader.getTermVector(doc_id, field_name)

    def leaves(self):
        return self._reader.leaves()

class Searcher(LuceneWrapper):
    def __init__(self, **kwargs):
        super(Searcher, self).__init__(**kwargs)
        self._get_searcher()

    def search(self, query, limit=1000, offset=0, sort=None):
        if sort:
            col = TopFieldCollector.create(sort.sort, limit + offset,
                                           True, False, False, True)
        else:
            col = TopScoreDocCollector.create(limit + offset, True)

        self._searcher.search(query.query, col)
        docs = col.topDocs(offset, limit)
        res = SearchResults(docs, self._searcher)

        return res

    def raw_search(self, query, max_result=1000):
        docs = self._searcher.search(query, max_result)
        return SearchResults(docs, self._searcher)

    def close(self):
        if self._reader is not None:
            self._reader.close()
        self._reader = None


class SearchResults(LuceneWrapper):
    def __init__(self, docs, searcher):
        self.docs = docs
        self.searcher = searcher
        self.total_hits = docs.totalHits
        self._iter = None

    def __getitem__(self, key):
        return ScoredDocument(self.docs.scoreDocs.__getitem__(key), self.searcher)
    
    def __iter__(self):
        self._iter = iter(self.docs.scoreDocs)
        return self

    def next(self):
        return self.__next__()

    def __next__(self):
        if not self._iter:
            self.__iter__()

        doc = next(self._iter)
        return ScoredDocument(doc, self.searcher)
        

class ScoredDocument(LuceneWrapper):
    def __init__(self, doc, searcher):
        super(ScoredDocument, self).__init__()
        self.doc = doc
        self._searcher_doc = searcher.doc(doc.doc)
        self.number = doc.doc
        self.score = doc.score
        self.searcher = searcher

    def __getattr__(self, name):
        val = self._searcher_doc.get(name)
        return val

    def get_fields(self):
        return [Field(x) for x in self._searcher_doc.getFields()]


class Field(LuceneWrapper):
    def __init__(self, field):
        super(Field, self).__init__()
        self._field = field
        self.name = field.name()
        self.value = field.stringValue()

    def get_tokens(self):
        stream = self._field.tokenStream(self._get_analyzer(), None)
        stream.reset()
        result = []
        while stream.incrementToken():
            term = stream.getAttribute(CharTermAttribute.class_)
            result.append(term.toString())
        return result


class BooleanQuery(LuceneWrapper):
    def __init__(self):
        super(BooleanQuery, self).__init__()
        self.query = _BooleanQuery()

    def add_must(self, query):
        self.query.add(query.query, BooleanClause.Occur.MUST)

    def add_must_not(self, query):
        self.query.add(query.query, BooleanClause.Occur.MUST_NOT)

    def add_should(self, query):
        self.query.add(query.query, BooleanClause.Occur.SHOULD)

    def set_minimum_nubmber_should_match(self, count):
        self.query.setMinimumNumberShouldMatch(count)

class TermQuery(LuceneWrapper):
    def __init__(self, field_name, query_term):
        super(TermQuery, self).__init__()

        term = Term(field_name, str(query_term))
        self.query = _TermQuery(term)

class IntRangeQuery(LuceneWrapper):
    def __init__(self, field_name, int_min, int_max=None, min_inclusive=True, max_inclusive=True):
        super(IntRangeQuery, self).__init__()

        if int_max is None:
            int_max = int_min

        self.query = NumericRangeQuery.newIntRange(field_name, int_min, int_max, min_inclusive, max_inclusive)


class Query(LuceneWrapper):
    def __init__(self, field_name, query_text):
        super(Query, self).__init__()

        parser = StandardQueryParser()
        parser.setAllowLeadingWildcard(True);
        parser.setAnalyzer(self._get_analyzer())
        parser.setDefaultOperator(StandardQueryConfigHandler.Operator.AND)
        
        try:
            self.query = parser.parse(query_text, field_name)
        except lucene.JavaError as e:
            raise QueryParseError(e.getJavaException().getMessage())

class QueryParseError(LuceneWrapperError):
    def __init__(self, message):
        self.message = message

class Sort(LuceneWrapper):
    BYTES = 1
    DOC = 2
    DOUBLE = 3
    FLOAT = 4
    INT = 5
    LONG = 6
    SCORE = 7
    STRING = 8
    STRING_VAL = 9

    def __init__(self, field_name, sort_type, reverse=False):
        super(Sort, self).__init__()
        self.sort = _Sort(SortField(field_name, self._convert_field_type(sort_type), reverse))

    def _convert_field_type(self, field_type):
        if field_type == self.BYTES:
            return SortField.Type.BYTES
        elif field_type == self.DOC:
            return SortField.Type.DOC
        elif field_type == self.DOUBLE:
            return SortField.Type.DOUBLE
        elif field_type == self.FLOAT:
            return SortField.Type.FLOAT
        elif field_type == self.INT:
            return SortField.Type.INT
        elif field_type == self.LONG:
            return SortField.Type.LONG
        elif field_type == self.SCORE:
            return SortField.Type.SCORE
        elif field_type == self.STRING:
            return SortField.Type.STRING
        elif field_type == self.STRING_VAL:
            return SortField.Type.STRING_VAL

        raise LuceneWrapperError()


class Highlighter(LuceneWrapper):
    def __init__(self, query, tag_begin, tag_end):
        super(Highlighter, self).__init__()

        # create formatter
        formatter = SimpleHTMLFormatter(tag_begin, tag_end)
        scorer = QueryScorer(query.query)
        self.highlighter = _Highlighter(formatter, scorer)
        self.analyzer = self._get_analyzer()

    def get_best_fragment(self, field, text, max_num_fragments=None):
        return self.highlighter.getBestFragment(self.analyzer, field, text)

    def get_best_fragments(self, field, text, max_num_fragments):
        return self.highlighter.getBestFragments(self.analyzer, field, text, max_num_fragments)

