# coding: utf-8

import mysql.connector

CONFIG_FILE="./.config.yml"

class NewslashDB(object):
    def __init__(self, config):
        self._config = config or {}
        self._conn = None

        # set default config values
        self._set_config_default("default_limit", 100)

    def config(self, key, value=None):
        if value:
            self._config[key] = value
            return value
        return self._config.get(key)

    def _set_config_default(self, key, value):
        if key not in self._config:
            self.config(key, value)

    def cursor(self):
        if not self._conn:
            self.connect()
        cur = self._conn.cursor(buffered=True)
        return cur

    def execute(self, query, **kwargs):
        cur = self.cursor()
        try:
            cur.execute(query, params=kwargs)
        except mysql.connector.ProgrammingError as e:
            raise NewslashDBProgrammingError(e)
        return cur

    def execute_multi(self, sqls, **kwargs):
        self.start_transaction()

        try:
            for sql in sqls:
                self.execute(sql, **kwargs)
        except NewslashDBError as e:
            self.rollback()
            raise e

        self.commit()

    def generic_insert(self, table_name, params, args):
        cols = ", ".join(params)
        values = ", ".join(["%({})s".format(x) for x in params])
        sql = "INSERT INTO {} ({}) VALUES ({})".format(table_name, cols, values)
        return self.execute(sql, **args)
    
    def generic_update(self, table_name, params, where_params, args):
        sets = ", ".join(["{}=%({})s".format(x, x) for x in params])
        wheres = " AND ".join(["{}=%({})s".format(x, x) for x in where_params])
        sql = "UPDATE {} SET {} WHERE {}".format(table_name, sets, wheres)
        return self.execute(sql, **args)

    def start_transaction(self):
        if not self._conn:
            self.connect()
        self._conn.start_transaction()
        return self

    def commit(self):
        self._conn.commit()
        return self

    def rollback(self):
        self._conn.rollback()
        return self

    def connect(self):
        if self.config("host").find(":") >= 0:
            (host, port) = self.config("host").split(":", 1)
            self._conn = mysql.connector.connect(user=self.config("user"),
                                                 password=self.config("password"),
                                                 database=self.config("name"),
                                                 host=host,
                                                 port=port)
        else:
            self._conn = mysql.connector.connect(user=self.config("user"),
                                                 password=self.config("password"),
                                                 database=self.config("name"),
                                                 host=self.config("host"))
        self._conn.autocommit = True
        return self

    def url(self):
        return 'mysql+mysqlconnector://{}:{}@{}/{}'.format(self.config("user"),
                                                           self.config("password"),
                                                           self.config("host"),
                                                           self.config("name"))
    def close(self):
        if self._conn:
            self._conn.close()
            self._conn = None
        return self

class NewslashDBError(Exception):
    def __str__(self):
        return self.message

class NewslashDBProgrammingError(NewslashDBError):
    def __init__(self, e):
        self.message = str(e)
        self.errno = e.errno


