# coding: utf-8
'''comments.py: newslash comment related database module'''

from base import NewslashDB, NewslashDBError
from datetime import datetime

CREATE_COMMENTS_TABLE_SQL = '''
CREATE TABLE IF NOT EXISTS `comments` (
  `sid`          mediumint(8) unsigned NOT NULL DEFAULT '0',
  `cid`          int(10) unsigned      NOT NULL AUTO_INCREMENT,
  `pid`          int(10) unsigned      NOT NULL DEFAULT '0',
  `date`         datetime              NOT NULL DEFAULT '1970-01-01 00:00:00',
  `last_update`  timestamp             NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  `ipid`         varchar(32)           NOT NULL DEFAULT '',
  `subnetid`     varchar(32)           NOT NULL DEFAULT '',
  `subject`      varchar(50)           NOT NULL DEFAULT '',
  `subject_orig` enum('no','yes')      NOT NULL DEFAULT 'yes',
  `uid`          mediumint(8) unsigned NOT NULL DEFAULT '0',
  `points`       tinyint(4)            NOT NULL DEFAULT '0',
  `pointsorig`   tinyint(4)            NOT NULL DEFAULT '0',
  `pointsmax`    tinyint(4)            NOT NULL DEFAULT '0',
  `lastmod`      mediumint(8) unsigned NOT NULL DEFAULT '0',
  `reason`       tinyint(3) unsigned   NOT NULL DEFAULT '0',
  `signature`    varchar(32)           NOT NULL DEFAULT '',
  `len`          smallint(5) unsigned  NOT NULL DEFAULT '0',
  `karma_bonus`  enum('yes','no')      NOT NULL DEFAULT 'no',
  `karma`        smallint(6)           NOT NULL DEFAULT '0',
  `karma_abs`    smallint(5) unsigned  NOT NULL DEFAULT '0',
  `tweak_orig`   tinyint(4)            NOT NULL DEFAULT '0',
  `tweak`        tinyint(4)            NOT NULL DEFAULT '0',
  `badge_id`     tinyint(3) unsigned   NOT NULL DEFAULT '0',
  PRIMARY KEY        (`cid`),
  KEY `display`      (`sid`,`points`,`uid`),
  KEY `byname`       (`uid`,`points`),
  KEY `ipid`         (`ipid`),
  KEY `subnetid`     (`subnetid`),
  KEY `theusual`     (`sid`,`uid`,`points`,`cid`),
  KEY `countreplies` (`pid`,`sid`),
  KEY `uid_date`     (`uid`,`date`),
  KEY `date_sid`     (`date`,`sid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 ROW_FORMAT=DYNAMIC
'''

CREATE_COMMENT_TEXT_TABLE_SQL = '''
CREATE TABLE IF NOT EXISTS `comment_text` (
  `cid`     int(10) unsigned NOT NULL DEFAULT '0',
  `comment` mediumtext       NOT NULL,
  PRIMARY KEY (`cid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 ROW_FORMAT=DYNAMIC
'''

class Comments(object):
    def __init__(self, config):
        self.db = NewslashDB(config)
        self._config = config

    def create_tables(self):
        '''this function implemented for test purpose...'''
        self.db.execute_multi((CREATE_COMMENTS_TABLE_SQL,
                               CREATE_COMMENT_TEXT_TABLE_SQL))

    def drop_tables(self):
        '''this function implemented for test purpose...'''
        self.db.execute_multi(('DROP TABLE comments',
                               'DROP TABLE comment_text'))

    def insert(self, **kwargs):
        '''this function implemented for test purpose...'''
        db = self.db.start_transaction()

        try:
            cur = db.generic_insert("comments", ("sid", "pid", "uid"), kwargs)
            kwargs["cid"] = cur.lastrowid
            db.generic_insert("comment_text", ("cid", "comment"), kwargs)
        except NewslashDBError as e:
            db.rollback()
            raise e

        db.commit()
        return kwargs["cid"]

    def select(self, **kwargs):
        if not "limit" in kwargs:
            kwargs["limit"] = self.db.config("default_limit")

        if not "offset" in kwargs:
            kwargs["offset"] = 0

        if "cid" in kwargs:
            query = ("SELECT comments.*, comment_text.* FROM comments"
                     "  LEFT JOIN comment_text USING(cid)"
                     "  WHERE comments.cid = %(cid)s"
                     "  LIMIT %(limit)s OFFSET %(offset)s")
        elif "last_update_ge" in kwargs:
            query = ("SELECT comments.*, comment_text.* FROM comments"
                     "  LEFT JOIN comment_text USING(cid)"
                     "  WHERE comments.cid <= %(cid_le)s AND comments.last_update >= %(last_update_ge)s"
                     "  ORDER BY comments.cid ASC LIMIT %(limit)s OFFSET %(offset)s")
        elif "cid_gt" in kwargs:
            query = ("SELECT comments.*, comment_text.* FROM comments"
                     "  LEFT JOIN comment_text USING(cid)"
                     "  WHERE comments.cid > %(cid_gt)s"
                     "  ORDER BY comments.cid ASC LIMIT %(limit)s OFFSET %(offset)s")
        else:
            query = ("SELECT comments.*, comment_text.* FROM comments"
                     "  LEFT JOIN comment_text USING(cid)"
                     "  ORDER BY comments.cid DESC LIMIT %(limit)s OFFSET %(offset)s")

        cur = self.db.execute(query, **kwargs)
        col_names = cur.column_names
        result = [dict(zip(col_names, x)) for x in cur]

        self.db.close()
        return result
