# -*- coding: utf-8 -*-
"""index: index model for newslash search system"""

import calendar
from datetime import datetime

import lucene_wrapper
import lucene
import exceptions
import htmlutil

def _to_timestamp(dt):
    '''convert datetime object to unixtime'''
    return calendar.timegm(dt.utctimetuple())
    

class Document(object):
    def __init__(self, doc_type, item):
        self.doc_type = doc_type
        self.indexable = True

        if doc_type == 'story':
            self.doc = self._make_story_document(item)
        elif doc_type == 'comment':
            self.doc = self._make_comment_document(item)
        elif doc_type == 'journal':
            self.doc = self._make_journal_document(item)
        elif doc_type == 'submission':
            self.doc = self._make_submission_document(item)
        elif doc_type == 'poll':
            self.doc = self._make_poll_document(item)
        elif doc_type == 'user':
            self.doc = self._make_user_document(item)
        else:
            raise exceptions.DocumentMakingError("invalid document type")

    def _make_story_document(self, item):
        '''make Document object from story object'''
        doc = lucene_wrapper.Document()

        # some item have invalid time data
        if item["time"] is None:
            raise exceptions.DocumentMakingError("'time' property is None")

        # convert datetime to UNIX timestamp
        timestamp = _to_timestamp(item["time"])
        last_update = _to_timestamp(item["last_update"])

        # prepare intro-/body-text, url
        introtext = item["introtext"] or ""
        bodytext = item["bodytext"] or ""
        (content_text, urls) = htmlutil.strip_html_tag(introtext + bodytext)

        doc.add_string_field("type", "story")
        doc.add_int_field("unique_id", item["stoid"])
        doc.add_string_field("id", item["sid"])

        doc.add_text_field("title", item["title"])
        doc.add_text_field("content_text", content_text)
        doc.add_text_field("dept", item["dept"])

        doc.add_int_field("create_time", timestamp)
        doc.add_int_field("last_update", last_update)
        doc.add_int_field("topic", item["tid"])
        doc.add_int_field("author", item["uid"])
        doc.add_int_field("submitter", item["submitter"])

        for url in urls:
            doc.add_string_field("url", url)

        self.unique_id = item["stoid"]
        if item["neverdisplay"] == "1":
            self.indexable = False

        return doc
    
    def _make_comment_document(self, item):
        '''make Document object from comment object'''
        doc = lucene_wrapper.Document()

        # convert datetime to UNIX timestamp
        timestamp = _to_timestamp(item["date"])
        last_update = _to_timestamp(item["last_update"])

        # prepare intro-/body-text, url
        (content_text, urls) = htmlutil.strip_html_tag(item["comment"])

        doc.add_string_field("type", "comment")
        doc.add_int_field("unique_id", item["cid"])
        doc.add_string_field("id", str(item["cid"]))

        doc.add_text_field("title", item["subject"])
        doc.add_text_field("content_text", content_text)

        doc.add_int_field("create_time", timestamp)
        doc.add_int_field("last_update", last_update)
        doc.add_int_field("author", item["uid"])
        doc.add_int_field("points", item["points"])
        

        for url in urls:
            doc.add_string_field("url", url)

        self.unique_id = item["cid"]
        return doc
    
    def _make_journal_document(self, item):
        '''make Document object from journal object'''
        doc = lucene_wrapper.Document()

        # convert datetime to UNIX timestamp
        timestamp = _to_timestamp(item["date"])
        last_update = _to_timestamp(item["last_update"])

        # prepare intro-/body-text, url
        text = item["article"] or ""
        (content_text, urls) = htmlutil.strip_html_tag(text)

        try:
            doc.add_string_field("type", "journal")
            doc.add_int_field("unique_id", item["id"])
            doc.add_string_field("id", str(item["id"]))

            doc.add_text_field("title", item["description"])
            doc.add_text_field("content_text", content_text)

            doc.add_int_field("create_time", timestamp)
            doc.add_int_field("last_update", last_update)
            doc.add_int_field("topic", item["tid"])
            doc.add_int_field("author", item["uid"])

            for url in urls:
                if url is not None:
                    doc.add_string_field("url", url)
        except lucene.InvalidArgsError:
            print("error:", item)
            raise

        self.unique_id = item["id"]
        if item["public"] == "no":
            self.indexable = False

        return doc

    def _make_submission_document(self, item):
        '''make Document object from submission object'''
        doc = lucene_wrapper.Document()

        # convert datetime to UNIX timestamp
        timestamp = _to_timestamp(item["time"])
        last_update = _to_timestamp(item["last_update"])

        # prepare intro-/body-text, url
        text = item["story"] or ""
        (content_text, urls) = htmlutil.strip_html_tag(text)

        doc.add_string_field("type", "submission")
        self.unique_id = item["subid"]
        doc.add_int_field("unique_id", self.unique_id)
        doc.add_string_field("id", str(item["subid"]))

        doc.add_text_field("title", item["subj"])
        doc.add_text_field("content_text", content_text)

        doc.add_int_field("create_time", timestamp)
        doc.add_int_field("last_update", last_update)
        doc.add_int_field("topic", item["tid"])
        doc.add_int_field("author", item["uid"])

        for url in urls:
            doc.add_string_field("url", url)

        if item["public"] == "no":
            self.indexable = False

        return doc

    def _make_poll_document(self, item):
        '''make Document object from poll object'''
        doc = lucene_wrapper.Document()

        # convert datetime to UNIX timestamp
        timestamp = _to_timestamp(item["date"])
        last_update = _to_timestamp(item["date"])

        # prepare intro-/body-text, url
        text = " ".join([x["answer"] for x in item["answers"]])
        (content_text, urls) = htmlutil.strip_html_tag(text)

        doc.add_string_field("type", "poll")
        self.unique_id = item["qid"]
        doc.add_int_field("unique_id", self.unique_id)
        doc.add_string_field("id", str(item["qid"]))

        doc.add_text_field("title", item["question"])
        doc.add_text_field("content_text", content_text)

        doc.add_int_field("create_time", timestamp)
        doc.add_int_field("last_update", last_update)
        doc.add_int_field("topic", item["topic"])
        doc.add_int_field("author", item["uid"])

        for url in urls:
            doc.add_string_field("url", url)

        if item["polltype"] == "nodisplay":
            self.indexable = False

        return doc

    def _make_user_document(self, item):
        '''make Document object from poll object'''
        doc = lucene_wrapper.Document()

        # convert datetime to UNIX timestamp
        timestamp = _to_timestamp(item["created_at"])
        last_update = _to_timestamp(item["created_at"])

        # prepare intro-/body-text, url
        for k in ("realname", "bio", "homepage", "sig"):
            if item[k] is None:
                item[k] = ""
        content_text = item["realname"] + item["bio"] + item["homepage"] + item["sig"]

        doc.add_string_field("type", "user")
        self.unique_id = item["uid"]
        doc.add_int_field("unique_id", self.unique_id)
        doc.add_string_field("id", str(item["uid"]))

        doc.add_text_field("title", item["nickname"])
        doc.add_text_field("content_text", content_text)

        doc.add_int_field("create_time", timestamp)
        doc.add_int_field("last_update", last_update)

        return doc
