# -*- coding: utf-8 -*-
"""metadata: index metadata for newslash search system"""

import newslash_db
import sqlite3
import os.path
#from mysql.connector.errors import ProgrammingError
import exceptions

class SQLiteDB(object):
    def __init__(self, path):
        self._path = path
        self._conn = None

    def connect(self):
        self._conn = sqlite3.connect(self._path);
        #self._conn.autocommit = True
        return self

    def cursor(self):
        if not self._conn:
            self.connect()
        cur = self._conn.cursor()
        return cur

    def execute(self, query, **kwargs):
        cur = self.cursor()
        cur.execute(query, kwargs)
        return cur

    def commit(self):
        self._conn.commit()
        return self

    def rollback(self):
        self._conn.rollback()
        return self

    def close(self):
        if self._conn:
            self.commit()
            self._conn.close()
            self._conn = None
        return self


class Metadata(object):
    def __init__(self, config):
        self._config = config
        path = config.get("index_path")
        if not path:
            raise exceptions.ConfigFileError(exceptions.ConfigFileError.PARAMETER_NOT_FOUND, "index_path")
            
        self._db_path = os.path.join(path, "_ns_metadata.db");

    def _get_db(self):
        return SQLiteDB(self._db_path);
    
    def get_current_timestamp(self):
        db = self._get_db()
        cur = db.execute("SELECT datetime('now')")
        r = cur.fetchone()[0]
        db.close()
        return r

    def create_table(self):
        sql = (
            "CREATE TABLE ns_search_metadata ("
            "  target_id    INTEGER PRIMARY KEY AUTOINCREMENT,"
            "  target_name  TEXT    NOT NULL UNIQUE,"
            "  last_update  TEXT    NOT NULL DEFAULT CURRENT_TIMESTAMP,"
            "  latest_id    INTEGER NOT NULL DEFAULT 0"
            ")"
        )

        db = self._get_db()
        try:
            db.execute(sql)
        except sqlite3.OperationalError as e:
            db.close()
            raise exceptions.DatabaseError('table creation error: {}'.format(str(e)))

        db.close()

    def drop_table(self):
        db = self._get_db()
        db.execute('DROP TABLE ns_search_metadata')
        db.close()

    def delete_all(self):
        db = self._get_db()
        db.execute('DELETE FROM ns_search_metadata')
        db.close()

    def get(self, target):
        sql = 'SELECT * from ns_search_metadata WHERE target_name = :target'
        db = self._get_db()
        cur = db.execute(sql, target=target)
        column_names = [x[0] for x in cur.description]

        row = cur.fetchone()
        if row and len(row) > 0:
            result = dict(zip(column_names, row))
        else:
            result = {}

        db.close()
        return result

    def get_all(self):
        sql = 'SELECT * from ns_search_metadata'
        db = self._get_db()
        cur = db.execute(sql)
        column_names = [x[0] for x in cur.description]
        result = [dict(zip(column_names, x)) for x in cur.fetchall()]
        db.close()
        return result
        

    def update(self, target, last_update, latest_id):
        # sqlite 3.24 or after, can use ON CONFLICT, but current package's version
        # not supported...
        # if latest_id > 0:
        #     sql = (
        #         "INSERT INTO ns_search_metadata"
        #         "  (target_name, last_update, latest_id)"
        #         "  VALUES (:target, :last_update, :latest_id)"
        #         "  ON CONFLICT (target_name) DO UPDATE SET"
        #         "    last_update = :last_update,"
        #         "    latest_id = :latest_id"
        #     )
        # else:
        #     sql = (
        #         "INSERT INTO ns_search_metadata"
        #         "  (target_name, last_update, latest_id)"
        #         "  VALUES (:target, :last_update, :latest_id)"
        #         "  ON CONFLIT (target_name) DO UPDATE SET"
        #         "    last_update = :last_update"
        #     )
        if latest_id > 0:
            update_sql = (
                "UPDATE OR IGNORE ns_search_metadata"
                "  SET last_update = :last_update,"
                "      latest_id = :latest_id"
                "  WHERE target_name = :target"
            )
            insert_sql = (
                "INSERT OR IGNORE INTO ns_search_metadata"
                "  (target_name, last_update, latest_id)"
                "  VALUES (:target, :last_update, :latest_id)"
            )
        else:
            update_sql = (
                "UPDATE OR IGNORE ns_search_metadata"
                "  SET last_update = :last_update,"
                "  WHERE target_name = :target"
            )
            insert_sql = (
                "INSERT OR IGNORE INTO ns_search_metadata"
                "  (target_name, last_update, latest_id)"
                "  VALUES (:target, :last_update, :latest_id)"
            )

        db = self._get_db()
        try:
            cur = db.execute(insert_sql, target=target, last_update=last_update, latest_id=latest_id)
            cur = db.execute(update_sql, target=target, last_update=last_update, latest_id=latest_id)
        except sqlite3.OperationalError as e:
            db.close()
            raise exceptions.DatabaseError('ns_search_metadata update error: {}'.format(str(e)))

        db.close()

