# -*- coding: utf-8
''' searchd: Search daemon for Newslash'''

import re
import sys
import os
import os.path

from yaml import load
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from newslash_searchd import Router, Route
import lucene_wrapper

lucene_wrapper.init_vm()

class SearchdError(Exception):
    def __init__(self, message):
        self.message = message


# load config file
def _load_config(pathname):
    try:
        fh = open(pathname)
    except IOError:
        raise SearchdError("config file not found")

    config = load(fh, Loader=Loader)
    fh.close()

    # convert relative index_path to absolute path
    section = config.get('Searchd')
    if section: 
        index_path = section.get('index_path', '')
        if index_path[0] != '/':
            cfg_dir = os.path.dirname(pathname)
            index_path = os.path.normpath(os.path.join(cfg_dir, index_path))
            section["index_path"] = index_path
        
    return config

config_path = os.environ.get("SEARCHD_CONFIG", "/etc/newslash/searchd.conf")
if not os.path.exists(config_path):
    base_dir = os.path.dirname(os.path.realpath(__file__))
    config_path = os.path.join(base_dir, 'searchd.conf')
    if not os.path.exists(config_path):
        raise SearchdError("config file not found")

config = _load_config(config_path)


class Searchd(Router):
    def __init__(self, environ, start_response):
        super(Searchd, self).__init__(environ, start_response)
        self.default_route(Root())
        self.route(re.compile(r'^/admin'), SearchdAdmin())


class Root(Route):
    def __init__(self):
        self._config = config
        
    def config(self, section, key=None, default=None):
        if key is None:
            return self._config.get(section, default)
        d = self._config.get(section, {})
        return d.get(key, default)

    def get(self, req, resp):
        resp.render(200, json={"error": 0})

    def html_escape(self, text):
        text = text.replace("&", "&amp;")
        text = text.replace("<", "&lt;")
        text = text.replace(">", "&gt;")
        return text;
        
    def post(self, req, resp):
        # check request body is valid
        if req.body is None:
            resp.render(400)
            return

        # start query
        query_text = req.body.get('query', '')
        limit = req.body.get('limit', 10)
        target = req.body.get('target', 'story')
        sort_key = req.body.get('sort_key', 'create_time')
        sort_reverse = req.body.get('sort_reverse', 1) == 1

        try:
            offset = int(req.body.get('offset', 0))
        except:
            offset = 0

        # log
        #if req.environ['wsgi.errors']:
        #req.environ['wsgi.errors'].write(query.encode('utf-8'))

        index_dir = os.path.join(self.config('Searchd', 'index_path'), 'lucene_index')
        searcher = lucene_wrapper.Searcher(index_directory=index_dir)
        try:
            content_query = lucene_wrapper.Query("content_text", query_text)
            title_query = lucene_wrapper.Query("title", query_text)
            query = lucene_wrapper.BooleanQuery()
            query.set_minimum_nubmber_should_match(1)
            query.add_should(content_query)
            query.add_should(title_query)

            if target != 'all':
                target_query = lucene_wrapper.TermQuery("type", target)
                query.add_must(target_query)
        except lucene_wrapper.QueryParseError as e:
            resp.render(400, json={"error": { "message": e.message }})
            searcher.close()
            return
        except Exception as e:
            resp.render(500, json={"error": { "message": "query_error" }})
            req.environ['wsgi.errors'].write("query error: {} - query is {} ".format(e, query_text))
            searcher.close()
            return

        try:
            sort = lucene_wrapper.Sort(sort_key, lucene_wrapper.Sort.INT, sort_reverse)
            result = searcher.search(query, limit, offset, sort)
        except Exception as e:
            resp.render(500, json={"error": { "message": "search_error" }})
            req.environ['wsgi.errors'].write("search error: {} - query is {} \n".format(e, query_text))
            searcher.close()
            return

        resp_body = {
            "total_hits": result.total_hits,
            "hits": [],
            "start": offset,
        }

        # for highlighting
        highlighter = lucene_wrapper.Highlighter(query, "<strong>", "</strong>")

        for item in result:
            texts = highlighter.get_best_fragments("content_text", self.html_escape(item.content_text), 2)
            content_text = "".join(texts)
            if content_text is None or len(texts) == 0:
                content_text = item.content_text

            title = highlighter.get_best_fragment("title", item.title)
            if title is None or len(title) == 0:
                title = item.title

            resp_body["hits"].append({ "number": item.number,
                                       "type": item.type,
                                       "id": item.id,
                                       "title": title,
                                       "author": item.author,
                                       "create_time": item.create_time,
                                       "content_text": content_text,
            })

        # done
        searcher.close()
        resp.render(200, json=resp_body)
    

class SearchdAdmin(Root):
    def get(self, req, resp):
        resp.render(200, json={"error": 0})
        
    def post(self, req, resp):
        pass


if __name__ == '__main__':
    from wsgiref.simple_server import make_server, WSGIRequestHandler

    host = config.get('host', "")
    port = config.get('port', 6000)
    if len(host) > 0 and host.find(":") >= 0:
        (host, port) = host.split(":", 1)

    server = make_server(host, port, Searchd)

    print("starting server at {}:{}...".format(host, port))
    server.serve_forever()
