#!/usr/bin/perl
use strict;
use warnings;
use utf8;
use feature ':5.10';

use Getopt::Std;
use DBI;
use JSON qw(decode_json);
use IO::File;
use File::Basename;
use File::Spec;

use constant MESSAGE_TYPES_TABLE_NAME => 'ns_message_types';
use constant MESSAGE_TYPES_SETTINGS => 'message_types.json';

use Data::Dumper;

my $PROGNAME = basename($0);
my $VERSION = '0.1.0';

sub main {
    my $opts = {};

    if (!getopts( "hvru:H:p:d:", $opts)) {
        usage('Options used incorrectly.');
    }

    if ($opts->{'h'}) {
        usage();
    }
    my $input_file = shift @ARGV;
    usage() if !$input_file;

    my $dbh = connect_db($opts);
    if (!$dbh) {
        usage('cannot connect to database.');
    }

    create_table($dbh);
    insert_message_type($dbh, $input_file);

    $dbh->disconnect;
    print "insert done.\n";

}

sub connect_db {
  my $opts = shift @_;

  my $host = $opts->{'H'} || "";
  my $user = $opts->{'u'} || "";
  my $password = $opts->{'p'} || "";
  my $name = $opts->{'d'} || "";

  my $dbh = DBI->connect("DBI:mysql:$name:$host", $user, $password);
  return if !$dbh;
  $dbh->do('SET NAMES utf8mb4');
  return $dbh;
}

sub usage {
  print "*** $_[0] \n" if $_[0];
  print <<EOT;

Usage: $PROGNAME [OPTIONS] <input json file>

This will read datas from json file and insert them to ns_message_types table.

Main options:
      -h    Help (this message)
      -v    Version
      -u    Username to use when connecting to server
      -H    Connect to host
      -p    Password to use when connecting to server
      -d    The database to use
EOT
  exit;
}

sub create_table {
    my $dbh = shift;
    my $table_name = MESSAGE_TYPES_TABLE_NAME;

    my $sql = <<"EOSQL";
CREATE TABLE IF NOT EXISTS $table_name (
  message_type_id     int(11)     NOT NULL PRIMARY KEY,
  name                varchar(32) NOT NULL,
  description         varchar(32) NOT NULL,
  seclev              int(11)     NOT NULL DEFAULT '1',
  selectable_mode     int(11)     NOT NULL DEFAULT '1'
) DEFAULT CHARSET=utf8mb4
EOSQL
    my $rs = $dbh->do($sql, undef);

    if ($rs) {
        return 1;
    }
    print STDERR "table $table_name not created.\n";
    return;
}

sub insert_message_type {
    my $dbh = shift;
    my $json_file = shift;
    my $table_name = MESSAGE_TYPES_TABLE_NAME;

    my $fh = IO::File->new($json_file, "r");
    return if !$fh;
    my $json_text = do { local $/; <$fh> };
    $fh->close;
    my $types = decode_json($json_text);

    my $sql = <<"EOSQL";
INSERT INTO $table_name 
    (message_type_id, name, description, seclev, selectable_mode)
  VALUES
    (?,               ?,    ?,           ?,      ?)
EOSQL
    $dbh->{AutoCommit} = 0;
    my $sth = $dbh->prepare($sql);
    for my $item (@{$types->{items}}) {
        my $rs = $sth->execute($item->{id},
                               $item->{name},
                               $item->{description},
                               $item->{seclev},
                               $item->{selectable});
        if (!$rs) {
            warn "database error: " . $dbh->errstr;
        }
    }
    $dbh->commit;
    $dbh->{AutoCommit} = 1;
}

sub version {
  print <<EOT;

Usage: $PROGNAME [OPTIONS] <input json file>

This will read datas from json file and insert them to ns_message_types table.

Main options:
      -h    Help (this message)
      -v    Version
      -u    Username to use when connecting to server
      -H    Connect to host
      -p    Password to use when connecting to server
      -d    The database to use
EOT
  exit;
}

main();

__END__
